"""
Created on 15 Nov 2010

@author: hannes
"""

import unittest
import os.path
import ibmmq as mq
from ibmmq import CMQC

import test_setup

# pylint: disable=line-too-long, broad-exception-caught

class TestRFH2PutGet(test_setup.Tests):
    """This test case tests the RFH2 class and its methods.
    """

    messages_dir = os.path.join(os.path.dirname(__file__), "messages")

    def setUp(self):
        """ Create a new queue manager (TESTPMQI).
        Must be run as a user that has 'mqm' access.

        """
        super().setUp()

        with open(os.path.join(self.messages_dir, "single_rfh2.dat"), "rb") as file:
            self.single_rfh2_message = file.read()
        self.single_rfh2_message_not_well_formed = \
            self.single_rfh2_message[0:117] + self.single_rfh2_message[121:]

        with open(os.path.join(self.messages_dir, "multiple_rfh2.dat"), "rb") as file:
            self.multiple_rfh2_message = file.read()
        self.multiple_rfh2_message_not_well_formed = \
            self.multiple_rfh2_message[0:117] + self.multiple_rfh2_message[121:]

        self.create_queue(self.queue_name)

        self.put_queue = mq.Queue(self.qmgr, self.queue_name)
        self.get_queue = mq.Queue(self.qmgr, self.queue_name)
        self.clear_queue(self.get_queue)

    def tearDown(self):
        """ Delete queue manager (TESTPMQI).

        """
        self.put_queue.close()
        self.get_queue.close()
        self.delete_queue(self.queue_name)
        super().tearDown()

    @staticmethod
    def clear_queue(queue):
        """Clear a queue via repeated gets"""
        try:
            while True:
                queue.get()
        except mq.MQMIError as e:
            if e.reason == 2033:
                return
            raise e

    def test_get_rfh2_single(self):
        """Use get_rfh2 to get a known correct 3rd party message that contains a single RFH2 header.
        """
        try:

            put_mqmd = mq.md()
            put_mqmd["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_mqmd["Encoding"] = 273
            put_mqmd["CodedCharSetId"] = 1208
            self.put_queue.put(self.single_rfh2_message, put_mqmd)

            get_mqmd = mq.md()
            get_opts = mq.gmo()
            rfh2_list = []
            msg = self.get_queue.get_rfh2(None, get_mqmd, get_opts, rfh2_list)

            self.assertEqual(len(rfh2_list), 1, "Number of RFH2's incorrect.  Should be one.  But is %i" % len(rfh2_list))

            rfh2 = rfh2_list[0]
            self.assertEqual(len(rfh2.get()), 14,
                             "Number of attributes incorrect.  Should be 12? But is %s"
                             % str(len(rfh2.get())))
            self.assertEqual(rfh2["_StrucId"], CMQC.MQRFH_STRUC_ID,
                             "StrucId has incorrect value. Should be: %s But is: %s"
                             % (CMQC.MQRFH_STRUC_ID, str(rfh2["_StrucId"])))
            self.assertEqual(rfh2["Version"], CMQC.MQRFH_VERSION_2,
                             "Version has incorrect value. Should be: %i But is: %s"
                             % (CMQC.MQRFH_VERSION_2, str(rfh2["Version"])))
            self.assertEqual(rfh2["StrucLength"], 284,
                             "StrucLength has incorrect value. Should be: %i But is: %s"
                             % (284, str(rfh2["StrucLength"])))
            self.assertEqual(rfh2["Encoding"], 273,
                             "Encoding has incorrect value. Should be: %i But is: %s"
                             % (273, str(rfh2["Encoding"])))
            self.assertEqual(rfh2["CodedCharSetId"], 1208,
                             "CodedCharSetId has incorrect value. Should be: %i But is: %s"
                             % (1208, str(rfh2["CodedCharSetId"])))
            self.assertEqual(rfh2["Format"], CMQC.MQFMT_STRING,
                             "Format has incorrect value. Should be: %s But is: %s"
                             % (CMQC.MQFMT_NONE, str(rfh2["Format"])))
            self.assertEqual(rfh2["Flags"], 0,
                             "Flags has incorrect value. Should be: %i But is: %s"
                             % (0, str(rfh2["Flags"])))
            self.assertEqual(rfh2["NameValueCCSID"], 1208,
                             "NameValueCCSID has incorrect value. Should be: %i But is: %s"
                             % (1208, str(rfh2["NameValueCCSID"])))
            self.assertEqual(rfh2["pscLength"], 152,
                             "pscLength has incorrect value. Should be: %i But is: %s"
                             % (152, str(rfh2["pscLength"])))
            self.assertEqual(rfh2["psc"],
                             b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc> ",
                             "psc has incorrect value. Should be: %s But is: %s"
                             % ("<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc> ", ">" + str(rfh2["psc"]) + "<"))
            self.assertEqual(rfh2["testFolderLength"], 56,
                             "testFolderLength has incorrect value. Should be: %i But is: %s"
                             % (56, str(rfh2["testFolderLength"])))
            self.assertEqual(rfh2["testFolder"], b"<testFolder><testVar>testValue</testVar></testFolder>   ",
                             "testFolder has incorrect value. Should be: %s But is: %s"
                             % ("<testFolder><testVar>testValue</testVar></testFolder>   ", str(rfh2["testFolder"])))
            self.assertEqual(rfh2["mcdLength"], 28,
                             "mcdLength has incorrect value. Should be: %i But is: %s"
                             % (28, str(rfh2["mcdLength"])))
            self.assertEqual(rfh2["mcd"], b"<mcd><Msd>xmlnsc</Msd></mcd>",
                             "mcd has incorrect value. Should be: %s But is: %s"
                             % ("<mcd><Msd>xmlnsc</Msd></mcd>", str(rfh2["mcd"])))

            self.assertEqual(msg, self.single_rfh2_message[rfh2["StrucLength"]:], "Message Payloads do not match?")
        except Exception as e:
            self.fail(e)

    def test_get_rfh2_multiple(self):
        """Use get_rfh2 to get a known correct 3rd party message containing Multiples RFH2 headers.
        """

        try:

            put_mqmd = mq.md()
            put_mqmd["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_mqmd["Encoding"] = 273
            put_mqmd["CodedCharSetId"] = 1208
            self.put_queue.put(self.multiple_rfh2_message, put_mqmd)

            get_mqmd = mq.md()
            get_opts = mq.gmo()
            rfh2_list = []
            msg = self.get_queue.get_rfh2(None, get_mqmd, get_opts, rfh2_list)

            self.assertEqual(len(rfh2_list), 2, "Number of RFH2's incorrect.  Should be 2.  But is %i" % len(rfh2_list))

            rfh2_1 = rfh2_list[0]
            self.assertEqual(len(rfh2_1.get()), 12,
                             "Number of attributes incorrect.  Should be 12? But is %s"
                             % str(len(rfh2_1.get())))
            self.assertEqual(rfh2_1["_StrucId"], CMQC.MQRFH_STRUC_ID,
                             "StrucId has incorrect value. Should be: %s But is: %s"
                             % (CMQC.MQRFH_STRUC_ID, str(rfh2_1["_StrucId"])))
            self.assertEqual(rfh2_1["Version"], CMQC.MQRFH_VERSION_2,
                             "Version has incorrect value. Should be: %i But is: %s"
                             % (CMQC.MQRFH_VERSION_2, str(rfh2_1["Version"])))
            self.assertEqual(rfh2_1["StrucLength"], 252,
                             "StrucLength has incorrect value. Should be: %i But is: %s"
                             % (284, str(rfh2_1["StrucLength"])))
            self.assertEqual(rfh2_1["Encoding"], 273,
                             "Encoding has incorrect value. Should be: %i But is: %s"
                             % (273, str(rfh2_1["Encoding"])))
            self.assertEqual(rfh2_1["CodedCharSetId"], 1208,
                             "CodedCharSetId has incorrect value. Should be: %i But is: %s"
                             % (1208, str(rfh2_1["CodedCharSetId"])))
            self.assertEqual(rfh2_1["Format"], CMQC.MQFMT_RF_HEADER_2,
                             "Format has incorrect value. Should be: %s But is: %s"
                             % (CMQC.MQFMT_RF_HEADER_2, str(rfh2_1["Format"])))
            self.assertEqual(rfh2_1["Flags"], 0,
                             "Flags has incorrect value. Should be: %i But is: %s"
                             % (0, str(rfh2_1["Flags"])))
            self.assertEqual(rfh2_1["NameValueCCSID"], 1208,
                             "NameValueCCSID has incorrect value. Should be: %i But is: %s"
                             % (1208, str(rfh2_1["NameValueCCSID"])))
            self.assertEqual(rfh2_1["pscLength"], 152,
                             "pscLength has incorrect value. Should be: %i But is: %s"
                             % (152, str(rfh2_1["pscLength"])))
            self.assertEqual(rfh2_1["psc"],
                             b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc> ",
                             "psc has incorrect value. Should be: %s But is: %s"
                             % ("<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc> ", ">" + str(rfh2_1["psc"]) + "<"))
            self.assertEqual(rfh2_1["testFolderLength"], 56,
                             "testFolderLength has incorrect value. Should be: %i But is: %s"
                             % (56, str(rfh2_1["testFolderLength"])))
            self.assertEqual(rfh2_1["testFolder"], b"<testFolder><testVar>testValue</testVar></testFolder>   ",
                             "testFolder has incorrect value. Should be: %s But is: %s"
                             % ("<testFolder><testVar>testValue</testVar></testFolder>   ", str(rfh2_1["testFolder"])))

            rfh2_2 = rfh2_list[1]
            self.assertEqual(len(rfh2_2.get()), 14,
                             "Number of attributes incorrect.  Should be 14? But is %s"
                             % str(len(rfh2_2.get())))
            self.assertEqual(rfh2_2["_StrucId"], CMQC.MQRFH_STRUC_ID,
                             "StrucId has incorrect value. Should be: %s But is: %s"
                             % (CMQC.MQRFH_STRUC_ID, str(rfh2_2["_StrucId"])))
            self.assertEqual(rfh2_2["Version"], CMQC.MQRFH_VERSION_2,
                             "Version has incorrect value. Should be: %i But is: %s"
                             % (CMQC.MQRFH_VERSION_2, str(rfh2_2["Version"])))
            self.assertEqual(rfh2_2["StrucLength"], 284,
                             "StrucLength has incorrect value. Should be: %i But is: %s"
                             % (284, str(rfh2_2["StrucLength"])))
            self.assertEqual(rfh2_2["Encoding"], 273,
                             "Encoding has incorrect value. Should be: %i But is: %s"
                             % (273, str(rfh2_2["Encoding"])))
            self.assertEqual(rfh2_2["CodedCharSetId"], 1208,
                             "CodedCharSetId has incorrect value. Should be: %i But is: %s"
                             % (1208, str(rfh2_2["CodedCharSetId"])))
            self.assertEqual(rfh2_2["Format"], CMQC.MQFMT_STRING,
                             "Format has incorrect value. Should be: %s But is: %s"
                             % (CMQC.MQFMT_STRING, str(rfh2_2["Format"])))
            self.assertEqual(rfh2_2["Flags"], 0,
                             "Flags has incorrect value. Should be: %i But is: %s"
                             % (0, str(rfh2_2["Flags"])))
            self.assertEqual(rfh2_2["NameValueCCSID"], 1208,
                             "NameValueCCSID has incorrect value. Should be: %i But is: %s"
                             % (1208, str(rfh2_2["NameValueCCSID"])))
            self.assertEqual(rfh2_2["pscLength"], 152,
                             "pscLength has incorrect value. Should be: %i But is: %s"
                             % (152, str(rfh2_2["pscLength"])))
            self.assertEqual(rfh2_2["psc"],
                             b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc> ",
                             "psc has incorrect value. Should be: %s But is: %s"
                             % ("<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc> ", ">" + str(rfh2_2["psc"]) + "<"))
            self.assertEqual(rfh2_2["testFolderLength"], 56,
                             "testFolderLength has incorrect value. Should be: %i But is: %s"
                             % (56, str(rfh2_2["testFolderLength"])))
            self.assertEqual(rfh2_2["testFolder"], b"<testFolder><testVar>testValue</testVar></testFolder>   ",
                             "testFolder has incorrect value. Should be: %s But is: %s"
                             % ("<testFolder><testVar>testValue</testVar></testFolder>   ", str(rfh2_2["testFolder"])))
            self.assertEqual(rfh2_2["mcdLength"], 28,
                             "mcdLength has incorrect value. Should be: %i But is: %s"
                             % (28, str(rfh2_2["mcdLength"])))
            self.assertEqual(rfh2_2["mcd"], b"<mcd><Msd>xmlnsc</Msd></mcd>",
                             "mcd has incorrect value. Should be: %s But is: %s"
                             % ("<mcd><Msd>xmlnsc</Msd></mcd>", str(rfh2_2["mcd"])))

            self.assertEqual(msg, self.multiple_rfh2_message[rfh2_1["StrucLength"] + rfh2_2["StrucLength"]:], "Message Payloads do not match?")

        except Exception as e:
            self.fail(e)

    def test_put_rfh2_single(self):
        """Create and put a new rfh2 and use get to get it from the queue.  Compare it against know correct message.
        """
        try:

            put_mqmd = mq.md()
            put_mqmd["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_mqmd["Encoding"] = 273
            put_mqmd["CodedCharSetId"] = 1208

            put_opts = mq.pmo()

            put_rfh2 = mq.RFH2()
            put_rfh2["_StrucId"] = CMQC.MQRFH_STRUC_ID
            put_rfh2["Version"] = CMQC.MQRFH_VERSION_2
            put_rfh2["StrucLength"] = 188
            put_rfh2["Encoding"] = 273
            put_rfh2["CodedCharSetId"] = 1208
            put_rfh2["Format"] = CMQC.MQFMT_STRING
            put_rfh2["Flags"] = 0
            put_rfh2["NameValueCCSID"] = 1208
            put_rfh2.add_folder(b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc>")
            put_rfh2.add_folder(b"<testFolder><testVar>testValue</testVar></testFolder>")
            put_rfh2.add_folder(b"<mcd><Msd>xmlnsc</Msd></mcd>")
            put_msg = b"<testData><testVar>testValue</testVar></testData>"
            self.put_queue.put_rfh2(put_msg, put_mqmd, put_opts, [put_rfh2])

            get_mqmd = mq.md()
            get_opts = mq.gmo()
            get_msg = self.get_queue.get(None, get_mqmd, get_opts)

            self.assertEqual(get_msg, self.single_rfh2_message, "Message got from Queue does not match known correct RFH2 message.")
        except Exception as e:
            self.fail(e)

    def test_put_rfh2_multiple(self):
        """Create and put a new RFH2 message containing multiple RFH2 headers and use get to get it from the queue.  Compare it against know correct message.
        """
        try:

            put_mqmd = mq.md()
            put_mqmd["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_mqmd["Encoding"] = 273
            put_mqmd["CodedCharSetId"] = 1208

            put_opts = mq.pmo()

            put_rfh2_1 = mq.RFH2()
            put_rfh2_1["_StrucId"] = CMQC.MQRFH_STRUC_ID
            put_rfh2_1["Version"] = CMQC.MQRFH_VERSION_2
            put_rfh2_1["StrucLength"] = 188
            put_rfh2_1["Encoding"] = 273
            put_rfh2_1["CodedCharSetId"] = 1208
            put_rfh2_1["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_rfh2_1["Flags"] = 0
            put_rfh2_1["NameValueCCSID"] = 1208
            put_rfh2_1.add_folder(b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc>")
            put_rfh2_1.add_folder(b"<testFolder><testVar>testValue</testVar></testFolder>")

            put_rfh2_2 = mq.RFH2()
            put_rfh2_2["_StrucId"] = CMQC.MQRFH_STRUC_ID
            put_rfh2_2["Version"] = CMQC.MQRFH_VERSION_2
            put_rfh2_2["StrucLength"] = 188
            put_rfh2_2["Encoding"] = 273
            put_rfh2_2["CodedCharSetId"] = 1208
            put_rfh2_2["Format"] = CMQC.MQFMT_STRING
            put_rfh2_2["Flags"] = 0
            put_rfh2_2["NameValueCCSID"] = 1208
            put_rfh2_2.add_folder(b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc>")
            put_rfh2_2.add_folder(b"<testFolder><testVar>testValue</testVar></testFolder>")
            put_rfh2_2.add_folder(b"<mcd><Msd>xmlnsc</Msd></mcd>")

            put_msg = b"<testData><testVar>testValue</testVar></testData>"
            self.put_queue.put_rfh2(put_msg, put_mqmd, put_opts, [put_rfh2_1, put_rfh2_2])

            get_mqmd = mq.md()
            get_opts = mq.gmo()

            get_msg = self.get_queue.get(None, get_mqmd, get_opts)

            self.assertEqual(get_msg, self.multiple_rfh2_message, "Message got from Queue does not match known correct RFH2 message.")

        except Exception as e:
            self.fail(e)

    def test_put_get_rfh2_single(self):
        """Create and put a new rfh2 with put_rfh2 and use get_rfh2 to get it from the queue.  Compare it against know correct message.
        """
        try:

            put_mqmd = mq.md()
            put_mqmd["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_mqmd["Encoding"] = 273
            put_mqmd["CodedCharSetId"] = 1208

            put_opts = mq.pmo()

            put_rfh2 = mq.RFH2()
            put_rfh2["_StrucId"] = CMQC.MQRFH_STRUC_ID
            put_rfh2["Version"] = CMQC.MQRFH_VERSION_2
            put_rfh2["StrucLength"] = 188
            put_rfh2["Encoding"] = 273
            put_rfh2["CodedCharSetId"] = 1208
            put_rfh2["Format"] = CMQC.MQFMT_STRING
            put_rfh2["Flags"] = 0
            put_rfh2["NameValueCCSID"] = 1208
            put_rfh2.add_folder(b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc>")
            put_rfh2.add_folder(b"<testFolder><testVar>testValue</testVar></testFolder>")
            put_rfh2.add_folder(b"<mcd><Msd>xmlnsc</Msd></mcd>")
            put_msg = b"<testData><testVar>testValue</testVar></testData>"

            put_rfh2_list = [put_rfh2]
            self.put_queue.put_rfh2(put_msg, put_mqmd, put_opts, put_rfh2_list)

            get_mqmd = mq.md()
            get_opts = mq.gmo()
            get_rfh2_list = []
            get_msg = self.get_queue.get_rfh2(None, get_mqmd, get_opts, get_rfh2_list)

            self.assertEqual(len(get_rfh2_list), len(put_rfh2_list), "Number of RFH2's incorrect.  Should be %i.  But is %i" % (len(get_rfh2_list), len(put_rfh2_list)))
            self.assertEqual(get_rfh2_list[0].get(), put_rfh2_list[0].get(), "Put and Get RFH2 Lists do not match.")
            self.assertEqual(get_msg, put_msg, "Put and Get messages do not match.")

        except Exception as e:
            self.fail(e)

    def test_put_get_rfh2_multiple(self):
        """Create and put a new RFH2 message containing multiple RFH2 headers with put_rfh2 and use get_rfh2 to get it from the queue.  Compare it against know correct message.
        """

        try:
            put_mqmd = mq.md()
            put_mqmd["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_mqmd["Encoding"] = 273
            put_mqmd["CodedCharSetId"] = 1208

            put_opts = mq.pmo()

            put_rfh2_1 = mq.RFH2()
            put_rfh2_1["_StrucId"] = CMQC.MQRFH_STRUC_ID
            put_rfh2_1["Version"] = CMQC.MQRFH_VERSION_2
            put_rfh2_1["StrucLength"] = 188
            put_rfh2_1["Encoding"] = 273
            put_rfh2_1["CodedCharSetId"] = 1208
            put_rfh2_1["Format"] = CMQC.MQFMT_RF_HEADER_2
            put_rfh2_1["Flags"] = 0
            put_rfh2_1["NameValueCCSID"] = 1208
            put_rfh2_1.add_folder(b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc>")
            put_rfh2_1.add_folder(b"<testFolder><testVar>testValue</testVar></testFolder>")

            put_rfh2_2 = mq.RFH2()
            put_rfh2_2["_StrucId"] = CMQC.MQRFH_STRUC_ID
            put_rfh2_2["Version"] = CMQC.MQRFH_VERSION_2
            put_rfh2_2["StrucLength"] = 188
            put_rfh2_2["Encoding"] = 273
            put_rfh2_2["CodedCharSetId"] = 1208
            put_rfh2_2["Format"] = CMQC.MQFMT_STRING
            put_rfh2_2["Flags"] = 0
            put_rfh2_2["NameValueCCSID"] = 1208
            put_rfh2_2.add_folder(b"<psc><Command>RegSub</Command><Topic>$topictree/topiccat/topic</Topic><QMgrName>DebugQM</QMgrName><QName>PUBOUT</QName><RegOpt>PersAsPub</RegOpt></psc>")
            put_rfh2_2.add_folder(b"<testFolder><testVar>testValue</testVar></testFolder>")
            put_rfh2_2.add_folder(b"<mcd><Msd>xmlnsc</Msd></mcd>")
            put_rfh2_list = [put_rfh2_1, put_rfh2_2]
            put_msg = b"<testData><testVar>testValue</testVar></testData>"

            self.put_queue.put_rfh2(put_msg, put_mqmd, put_opts, put_rfh2_list)

            get_mqmd = mq.md()
            get_opts = mq.gmo()
            get_rfh2_list = []
            get_msg = self.get_queue.get_rfh2(None, get_mqmd, get_opts, get_rfh2_list)

            self.assertEqual(len(get_rfh2_list), len(put_rfh2_list), "Number of RFH2's incorrect.  Should be %i.  But is %i" % (len(get_rfh2_list), len(put_rfh2_list)))
            self.assertEqual(get_rfh2_list[0].get(), put_rfh2_list[0].get(), "Put and Get RFH2 Lists do not match.")
            self.assertEqual(get_rfh2_list[1].get(), put_rfh2_list[1].get(), "Put and Get RFH2 Lists do not match.")
            self.assertEqual(get_msg, put_msg, "Put and Get messages do not match.")

        except Exception as e:
            self.fail(e)


if __name__ == '__main__':
    unittest.main()



# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example looks at the installed packages on an RPM-based Linux
machine and displays information about the MQ Client, if it is installed
using the default package name.
"""

import logging

import rpm

logging.basicConfig(level=logging.INFO)

package_name = 'MQSeriesClient'

ts = rpm.TransactionSet()
mi = ts.dbMatch('name', package_name)

if not mi.count():
    logging.info('Did not find package `%s` in RPM database.', package_name)
else:
    for header in mi:
        version = header['version']
        logging.info('Found package `%s`, version `%s`.', package_name, version)



"""MQCNO: Connection Options"""

# Copyright (c) 2025 IBM Corporation and other Contributors. All Rights Reserved.
# Copyright (c) 2009-2024 Dariusz Suchojad. All Rights Reserved.

from mqcommon import *
from mqopts import MQOpts
from ibmmq import CMQC, ibmmqc

class CNO(MQOpts):
    """ Construct an MQCNO Structure with default values as per MQI.
    The default values may be overridden by the optional keyword arguments 'kw'.

    It assumes a minimum of MQNO_VERSION_5 (available from MQ 8.0) in the
    available fields. The connect() method will then set the version >5 if the
    application has not set it
    """
    def __init__(self, **kw):

        cno_current_version = ibmmqc.__strucversions__.get("cno", 1)
        opts = [['_StrucId', CMQC.MQCNO_STRUC_ID, '4s'],
                ['Version', CMQC.MQCNO_VERSION_5, MQLONG_TYPE],
                ['Options', CMQC.MQCNO_NONE, MQLONG_TYPE],
                ['_ClientConnOffset', 0, MQLONG_TYPE],
                ['_ClientConnPtr', 0, 'P'],
                ['ConnTag', b'', '128s'],
                ['_SSLConfigPtr', 0, 'P'],
                ['_SSLConfigOffset', 0, MQLONG_TYPE],
                ['ConnectionId', b'', '24s'],
                ['_SecurityParmsOffset', 0, MQLONG_TYPE],
                ['_SecurityParmsPtr', 0, 'P']]

        if cno_current_version >= CMQC.MQCNO_VERSION_6:
            opts += [['CCDTUrl', 0, 'P'],
                     ['_CCDTUrlOffset', 0, MQLONG_TYPE],
                     ['_CCDTUrlLength', 0, MQLONG_TYPE],
                     ['_Reserved', 0, INTEGER64_TYPE]]

        if cno_current_version >= CMQC.MQCNO_VERSION_7:
            opts += [['ApplName', CMQC.MQAN_NONE, '28s'],
                     ['_Reserved2', 0, MQLONG_TYPE]]

        if cno_current_version >= CMQC.MQCNO_VERSION_8:
            opts += [['_BalanceParmsPtr', 0, 'P'],
                     ['_BalanceParmsOffset', 0, MQLONG_TYPE],
                     ['_Reserved3', 0, MQLONG_TYPE]]

        super().__init__(tuple(opts), **kw)



# Python MQI Class Wrappers. High level classes for the MQI
# Extension. These present an object interface to the MQI.
#
# Author: L. Smithson (lsmithson@open-networks.co.uk)
# Author: Dariusz Suchojad (dsuch at zato.io)
# Author: Mark Taylor (ibmmqmet on GitHub)
#

# Copyright (c) 2025, 2026 IBM Corporation and other Contributors. All Rights Reserved.
# Copyright (c) 2009-2024 Dariusz Suchojad. All Rights Reserved.

"""
ibmmq - Python Classes for IBM MQ

These classes wrap the lower level C MQI calls. They present an OO
interface originally inspired by the MQI C++ classes.

Classes are also provided for easy use of the MQI structure parameters
(MQMD, MQGMO etc.) from Python. These allow Python scripts to set/get
structure members by attribute, dictionary etc.

External classes corresponding to MQI structures include:

    * MD              - Main MQI structures
    * MDE
    * CNO, CSP
    * BNO, SCO, CD
    * OD,  SD
    * GMO, PMO
    * SRO, STS

    * CBD, CTLO, CBC - Related to callbacks

    * CMHO, DMHO       - Message Properties
    * IMPO, SMPO, DMPO
    * PD

    * RFH2             - Application structures
    * DLH
    * TM, TMC2
    * IIH, CIH         - IMS, CICS headers

    * CFH              - PCF elements
    * CFGR
    * CFIL, CFIL64
    * CFIN, CFIN64
    * CFSF, CFBF, CFIF
    * CFSL, CFST
    * CFBS

Classes for MQ objects include
    * QueueManager
    * Queue
    * Topic
    * Subscription
    * MessageHandle

    * PCFExecute - Programmable Command Format operations

    * MQMIError - MQI specific error
    * PYIFError - Python library error


Other public classes include
    * OTelOptions - to control processing of OTel information in messages

The following MQI operations are supported:

    * MQCONN, MQDISC (QueueManager.connect()/QueueManager.disconnect())
    * MQCONNX (QueueManager.connectWithOptions())
    * MQOPEN/MQCLOSE (Queue.open(), Queue.close(), Topic.open(), Topic.close())
    * MQPUT/MQPUT1/MQGET (Queue.put(), QueueManager.put1(), Queue.get(), Topic.pub())
    * MQSTAT (QueueManager.stat())
    * MQCMIT/MQBACK (QueueManager.commit()/QueueManager.backout())
    * MQBEGIN (QueueManager.begin())
    * MQINQ (QueueManager.inquire(), Queue.inquire())
    * MQSET (Queue.set())
    * MQSUB/MQSUBRQ (Subscription.sub(), Subscription.subrq())
    * MQCRTMH/MQDLTMH (MessageHandle(), MessageHandle.dlt())
    * MQSETMP/MQINQMP/MQDLTMP (MessageHandle.properties.set(), MessageHandle.properties.inq(), MessageHandle.properties.dlt())

To use this package, connect to the Queue Manager (using QueueManager.connect()), then open a queue (using
Queue.open()). You may then put or get messages on the queue (using Queue.put(), Queue.get()), as required.

Where possible, this package assumes the MQI defaults for all parameters. It can also defer a queue open until the put/get call.

The library maps all MQI warning & error status to the MQMIError exception. Errors detected by the package itself raise
the PYIFError exception. Both these exceptions are subclasses of the Error class.

MQI constants are defined in the CMQC and CMQXC modules; PCF constants are defined in CMQCFC.

PCF commands and inquiries are executed by calling a MQCMD_* method on an instance of a PCFExecute object.

The package is thread safe. Objects have the same thread scope as their MQI counterparts.

"""

# This file has relatively little content but is the primary module applications will access directly.
# It loads the various classes and methods from other files in this directory.

# Get information about the loaded package
from importlib.metadata import version

# Python 3.8+ DLL loading for Windows only.
try:
    from os import add_dll_directory  # type: ignore
    from os import environ
    from os.path import join
    from os.path import exists

    mq_home_path = environ.get('MQ_FILE_PATH')

    if mq_home_path:
        for d in ['bin', 'bin64']:
            mq_dll_directory = join(mq_home_path, d)
            if exists(mq_dll_directory):
                add_dll_directory(mq_dll_directory)
except ImportError:
    # If it fails, we'll try to continue anyway
    pass

# Load the C library
try:
    from . import ibmmqc  # type: ignore
except ImportError:
    import ibmmqc  # type: ignore  # Backward compatibility

# Import the MQI definitions. They are not used directly in this
# file, but reexported for apps to reference.
from ibmmq import CMQCFC
from ibmmq import CMQXC, CMQSTRC

from ibmmq import CMQC

import mqlog

# Import the classes that implement each MQI structure. Order can be
# critical here to ensure we don't get loops in the definitions

# Using "import *" lets them get reexported as if they were part of this main module
from mqopts import *

from mqmd import *
from mqmde import *
from mqgmo import *
from mqpmo import *

from mqcbd import *

from mqcno import *
from mqbno import *
from mqcsp import *
from mqcd import *
from mqod import *
from mqsd import *

from mqsco import *
from mqsro import *
from mqsts import *

from mqctlo import *
from mqcbc import *

from mqtm import *
from mqxqh import *

from mqpcf import *
from mqrfh2 import *
from mqdlh import *
from mqiih import *
from mqcih import *

# The message property structures
from mqprops import *

# Exception classes
from mqerrors import *

# MQI Verbs associated with their relevant classes
from mqobject import *
from mqqargs import *  # A shared part
from mqqmgr import *
from mqqueue import *
from mqsub import *
from mqtopic import *
from mqmsghdl import *

# And the PCF parse/execute module
from mqadmin import *

# Import the OpenTelemetry propagator. If it fails,
# it's probably because OTel is not in use or configured for this
# environment. Ignore the error.
try:
    from mqotel import *
    otel_enabled = True
except ImportError:
    otel_enabled = False
mqlog.debug(f"__init__: otel_enabled = {otel_enabled}")

# The versions of this package (as found in the C and Python layers)
__c_version__ = ibmmqc.__version__
__py_version__ = version('ibmmq')
__version__ = __py_version__

# Ensure consistency. Messing with LD_LIBRARY_PATH for example might cause us to load the wrong
# extension version.
if __py_version__ != __c_version__:
    raise EnvironmentError(f'Mismatch between C \'{__c_version__}\' and Python \'{__py_version__}\' package versions')

# The CMDLEVEL the C code was compiled against
__cmdlevel__ = ibmmqc.__cmdlevel__

# Not really useful now, but left in for any compatibility
__mqbuild__ = ibmmqc.__mqbuild__

def get_versions():
    """Return an object containing versions for this library and the underlying MQ"""
    return {"module": __version__, "MQ": __cmdlevel__}

def to_string(v, encoding=EncodingDefault.bytes_encoding):
    """Use the specified encoding to convert MQCHAR[] to a Python3 string, stripping trailing NULs/spaces.
    If there's an error, return the input unchanged.
    """
    if isinstance(v, bytes):
        try:
            null_index = v.find(0)
            if null_index != -1:
                v = v[:null_index]
            return v.decode(encoding).strip()
        except UnicodeError:
            pass
    return v



# -*- coding: utf8 -*-
"""Test base API."""
from sys import version_info
import unittest

import ibmmq as mq

import utils
from test_setup import Tests

class TestGet(Tests):
    """Test Queue.get() method."""

    def setUp(self):
        """Initialize test environment."""
        super().setUp()

        self.create_queue(self.queue_name)

        self.message = b'12345'
        self.buffer_length = 3

        self.queue = mq.Queue(self.qmgr,
                                 self.queue_name,
                                 mq.CMQC.MQOO_INPUT_AS_Q_DEF | mq.CMQC.MQOO_OUTPUT)

    def tearDown(self):
        """Delete created objects."""
        if self.queue:
            self.queue.close()

        self.delete_queue(self.queue_name)

        super().tearDown()

    def _put_message(self):
        md = mq.MD()
        self.queue.put(self.message, md)

        return md

    ###########################################################################
    #
    # Real Tests start here
    #
    ###########################################################################

    def test_get_nontruncated(self):
        """Test nontruncated without buffer."""
        self._put_message()

        md_get = mq.MD()
        message = self.queue.get(None, md_get)

        self.assertEqual(self.message, message)

    def test_get_nontruncated_0(self):
        """Test nontruncated with zero buffer length."""
        self._put_message()

        md_get = mq.MD()
        try:
            self.queue.get(0, md_get)
        except mq.MQMIError as ex:
            self.assertEqual(ex.reason, mq.CMQC.MQRC_TRUNCATED_MSG_FAILED)
            self.assertEqual(ex.original_length,  # pylint: disable=no-member
                             len(self.message))

    def test_get_nontruncated_short(self):
        """Test nontruncated with short buffer."""
        self._put_message()

        md_get = mq.MD()
        try:
            self.queue.get(self.buffer_length, md_get)
        except mq.MQMIError as ex:
            self.assertEqual(ex.reason, mq.CMQC.MQRC_TRUNCATED_MSG_FAILED)
            self.assertEqual(ex.original_length,  # pylint: disable=no-member
                             len(self.message))

    def test_get_nontruncated_enough(self):
        """Test nontruncated with big enough buffer."""
        self._put_message()

        md_get = mq.MD()
        message = self.queue.get(len(self.message), md_get)

        self.assertEqual(self.message, message)

    def test_get_truncated(self):
        """Test truncated without buffer."""
        self._put_message()
        gmo = mq.GMO()
        gmo.Options = mq.CMQC.MQGMO_ACCEPT_TRUNCATED_MSG

        md_get = mq.MD()
        try:
            self.queue.get(0, md_get, gmo)
        except mq.MQMIError as ex:
            self.assertEqual(ex.reason, mq.CMQC.MQRC_TRUNCATED_MSG_ACCEPTED)
            self.assertEqual(ex.message, b'')  # pylint: disable=no-member
            self.assertEqual(ex.original_length,  # pylint: disable=no-member
                             len(self.message))

    def test_get_truncated_0(self):
        """Test truncated with zero buffer length."""
        self._put_message()
        gmo = mq.GMO()
        gmo.Options = mq.CMQC.MQGMO_ACCEPT_TRUNCATED_MSG

        md_get = mq.MD()
        try:
            self.queue.get(0, md_get, gmo)
        except mq.MQMIError as ex:
            self.assertEqual(ex.reason, mq.CMQC.MQRC_TRUNCATED_MSG_ACCEPTED)
            self.assertEqual(ex.message, b'')  # pylint: disable=no-member
            self.assertEqual(ex.original_length,  # pylint: disable=no-member
                             len(self.message))

    def test_get_truncated_short(self):
        """Test truncated with short buffer."""
        self._put_message()
        gmo = mq.GMO()
        gmo.Options = mq.CMQC.MQGMO_ACCEPT_TRUNCATED_MSG

        md_get = mq.MD()
        try:
            self.queue.get(self.buffer_length, md_get, gmo)
        except mq.MQMIError as ex:
            self.assertEqual(ex.reason, mq.CMQC.MQRC_TRUNCATED_MSG_ACCEPTED)
            self.assertEqual(ex.message,  # pylint: disable=no-member
                             self.message[:self.buffer_length])
            self.assertEqual(ex.original_length,  # pylint: disable=no-member
                             len(self.message))

    def test_get_truncated_enough(self):
        """Test truncated with big buffer."""
        self._put_message()
        gmo = mq.GMO()
        gmo.Options = mq.CMQC.MQGMO_ACCEPT_TRUNCATED_MSG

        md_get = mq.MD()
        message = self.queue.get(len(self.message), md_get, gmo)

        self.assertEqual(self.message, message)

    def test_get_nontruncated_big_msg(self):
        """Test get nontruncated big message"""
        md_put = mq.MD()
        if version_info.major >= 3:
            self.queue.put(bytes(4097), md_put)
        else:
            self.queue.put(bytes(b'\0' * 4097), md_put)

        md_get = mq.MD()
        message = self.queue.get(None, md_get)

        self.assertEqual(len(message), 4097)
        self.assertEqual(md_put.PutDate, md_get.PutDate)

    def test_get_truncated_big_msg(self):
        """Test get nontruncated big message"""
        md_put = mq.MD()
        if version_info.major >= 3:
            self.queue.put(bytes(4097), md_put)
        else:
            self.queue.put(bytes(b'\0' * 4097), md_put)
        gmo = mq.GMO()
        gmo.Options = mq.CMQC.MQGMO_ACCEPT_TRUNCATED_MSG

        md_get = mq.MD()
        try:
            _ = self.queue.get(None, md_get, gmo)
        except mq.MQMIError as ex:
            self.assertEqual(ex.reason, mq.CMQC.MQRC_TRUNCATED_MSG_ACCEPTED)
            self.assertEqual(ex.original_length,  # pylint: disable=no-member
                             4097)
            self.assertEqual(len(ex.message), 0)
            self.assertEqual(md_put.PutDate, md_get.PutDate)

    def test_put_string(self):
        """Test that putting a non-ASCII string is correctly converted during GET"""
        md = mq.MD()
        # file coding defined (utf-8)
        self.queue.put('тест', md)  # Cyrillic (non-ascii) characters

        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        # In Python3 can use unicode string
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put_string_with_ccsid_and_format(self):
        """Test putting a string with explicit ccsid"""
        md = mq.MD(
            CodedCharSetId=1208,  # coding: utf8 is set
            Format=mq.CMQC.MQFMT_STRING)

        self.queue.put('тест', md)  # Cyrillic (non-ascii) characters

        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        # In Python3 can use unicode string
        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put_unicode(self):
        """Put a string that's already encoded to unicode bytes"""
        self.queue.put('\u0442\u0435\u0441\u0442')  # Unicode characters

        md = mq.MD()
        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put_unicode_with_ccsid_and_format(self):
        """Put a string that's already encoded, and be explicit about the CCSID"""
        md = mq.MD(
            CodedCharSetId=1208,
            Format=mq.CMQC.MQFMT_STRING)

        self.queue.put('\u0442\u0435\u0441\u0442', md)  # Unicode characters

        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put1_bytes(self):
        """Use PUT1 to test string handling with pre-converted unicode bytes"""
        md = mq.MD()
        self.qmgr.put1(self.queue_name, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82', md)  # Non-ascii characters

        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_NONE)

    def test_put1_string(self):
        """Use PUT1 to test string handling with a unicode string"""
        md = mq.MD()
        # file coding defined (utf-8)
        self.qmgr.put1(self.queue_name, 'тест', md)  # Cyrillic (non-ascii) characters

        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        # In Python3 can use unicode string
        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put1_string_with_ccsid_and_format(self):
        """Use PUT1 to test string handling with a unicode string and explicit CCSID"""
        md = mq.MD(
            CodedCharSetId=1208,  # coding: utf8 is set
            Format=mq.CMQC.MQFMT_STRING)

        self.qmgr.put1(self.queue_name, 'тест', md)  # Cyrillic (non-ascii) characters

        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        # In Python3 can use unicode string
        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put1_unicode(self):
        """Use PUT1 to test string handling with a pre-converted unicode string"""
        self.qmgr.put1(self.queue_name, '\u0442\u0435\u0441\u0442')  # Unicode characters

        md = mq.MD()
        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put1_unicode_with_ccsid_and_format(self):
        """Use PUT1 to test string handling with a pre-converted unicode string and explicit CCSID"""
        md = mq.MD(
            CodedCharSetId=1208,
            Format=mq.CMQC.MQFMT_STRING)

        self.qmgr.put1(self.queue_name, '\u0442\u0435\u0441\u0442', md)  # Unicode characters

        gmo = mq.GMO()
        gmo.Options = gmo.Options & ~ mq.CMQC.MQGMO_CONVERT
        message = self.queue.get(None, md, gmo)

        self.assertEqual(message, b'\xd1\x82\xd0\xb5\xd1\x81\xd1\x82')
        self.assertEqual(md.Format, mq.CMQC.MQFMT_STRING)
        self.assertEqual(md.CodedCharSetId, 1208)

    def test_put1(self):
        """Simplest use of PUT1"""
        input_msg = b'Hello world!'
        self.qmgr.put1(self.queue_name, input_msg)
        # now get the message from the queue
        queue = mq.Queue(self.qmgr, self.queue_name)
        result_msg = queue.get()
        self.assertEqual(input_msg, result_msg)

    def test_inquire(self):
        """Test MQINQ on the qmgr"""
        attribute = mq.CMQC.MQCA_Q_MGR_NAME
        expected_value = self.queue_manager
        attribute_value = self.qmgr.inquire(attribute)
        self.assertEqual(len(attribute_value), mq.CMQC.MQ_Q_MGR_NAME_LENGTH)
        self.assertTrue(utils.strcmp(attribute_value.strip(), expected_value))


if __name__ == "__main__":
    unittest.main()



# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example shows putting a message to a queue using the put1 (MQPUT1) method
"""

import ibmmq as mq

queue_manager = "QM1"
channel = 'DEV.APP.SVRCONN'
host = '127.0.0.1'
port = '1414'
conn_info = '%s(%s)' % (host, port)
user = 'app'
password = 'password'

queue_name = 'DEV.QUEUE.1'
message = 'Hello from Python!'

qmgr = mq.connect(queue_manager, channel, conn_info, user, password)

qmgr.put1(queue_name, message)

qmgr.disconnect()

print("Done.")



"""MQRFH2 Structure: Name/Value pairs"""

# Copyright (c) 2025, 2026 IBM Corporation and other Contributors. All Rights Reserved.
# Copyright (c) 2009-2024 Dariusz Suchojad. All Rights Reserved.

import xml.etree.ElementTree as ET

from mqcommon import *
from mqopts import MQOpts
from mqerrors import *

from ibmmq import CMQC

try:
    from typing import Union, List
except ImportError:
    pass

# RFH2 Header parsing/creation Support - Hannes Wagener - 2010.
class RFH2(MQOpts):
    """ Construct a RFH2 Structure with default values as per MQI.
    The default values may be overridden by the optional keyword arguments 'kw'.
    """
    initial_opts = [['_StrucId', CMQC.MQRFH_STRUC_ID, '4s'],
                    ['Version', CMQC.MQRFH_VERSION_2, MQLONG_TYPE],
                    ['StrucLength', 0, MQLONG_TYPE],
                    ['Encoding', CMQC.MQENC_NATIVE, MQLONG_TYPE],
                    ['CodedCharSetId', CMQC.MQCCSI_Q_MGR, MQLONG_TYPE],
                    ['Format', CMQC.MQFMT_NONE, '8s'],
                    ['Flags', 0, MQLONG_TYPE],
                    ['NameValueCCSID', CMQC.MQCCSI_Q_MGR, MQLONG_TYPE]]  # type: List[List[Union[str, int, bytes]]]

    big_endian_encodings = [CMQC.MQENC_INTEGER_NORMAL,
                            CMQC.MQENC_DECIMAL_NORMAL,
                            CMQC.MQENC_FLOAT_IEEE_NORMAL,
                            CMQC.MQENC_FLOAT_S390,

                            # 17
                            CMQC.MQENC_INTEGER_NORMAL +
                            CMQC.MQENC_DECIMAL_NORMAL,

                            # 257
                            CMQC.MQENC_INTEGER_NORMAL +
                            CMQC.MQENC_FLOAT_IEEE_NORMAL,

                            # 272
                            CMQC.MQENC_DECIMAL_NORMAL +
                            CMQC.MQENC_FLOAT_IEEE_NORMAL,

                            # 273
                            CMQC.MQENC_INTEGER_NORMAL +
                            CMQC.MQENC_DECIMAL_NORMAL +
                            CMQC.MQENC_FLOAT_IEEE_NORMAL]

    def __init__(self, **kw):
        # Take a copy of private initial_opts
        self.opts = [list(x) for x in self.initial_opts]
        super().__init__(tuple(self.opts), **kw)

    def add_folder(self, folder_data):
        """ Adds a new XML folder to the RFH2 header.
        Checks if the XML is well formed and updates self.StrucLength.
        """

        ensure_not_unicode(folder_data)  # Python 3 bytes check

        # Check that the folder is valid xml and get the root tag name.
        try:
            folder_name = ET.fromstring(folder_data).tag
        except Exception as e:
            raise PYIFError('RFH2 - XML Folder not well formed. Exception: %s' % str(e)) from e

        # Make sure folder length divides by 4 - else add spaces
        folder_length = len(folder_data)
        remainder = folder_length % 4
        if remainder != 0:
            num_spaces = 4 - remainder
            folder_data = folder_data + b' ' * num_spaces
            folder_length = len(folder_data)

        self.opts.append([folder_name + 'Length', (folder_length), MQLONG_TYPE])
        self.opts.append([folder_name, folder_data, '%is' % folder_length])

        # Save the current values
        saved_values = self.get()

        # Reinit MQOpts with new fields added
        super().__init__(tuple(self.opts))

        # Reset the values to the saved values
        self.set(**saved_values)

        # Calculate the correct StrucLength
        self['StrucLength'] = self.get_length()

    def pack(self, encoding=None):
        """ Override pack in order to set correct numeric encoding in the format.
        """
        if encoding is not None:
            if encoding in self.big_endian_encodings:
                self.opts[0][2] = '>' + self.initial_opts[0][2]
                saved_values = self.get()

                # Apply the new opts
                super().__init__(tuple(self.opts))

                # Set from saved values
                self.set(**saved_values)

        return super().pack()

    def unpack(self, buff, encoding=None):
        """ Override unpack in order to extract and parse RFH2 folders.
        Encoding meant to come from the MQMD.
        """

        ensure_not_unicode(buff)  # Python 3 bytes check

        if buff[0:4] != CMQC.MQRFH_STRUC_ID:
            raise PYIFError('RFH2 - _StrucId not MQRFH_STRUC_ID. Value: %s' % buff[0:4])

        if len(buff) < 36:
            raise PYIFError('RFH2 - Buffer too short. Should be 36+ bytes instead of %s' % len(buff))
        # Take a copy of initial_opts and the lists inside
        self.opts = [list(x) for x in self.initial_opts]

        big_endian = False
        if encoding is not None:
            if encoding in self.big_endian_encodings:
                big_endian = True
        else:
            # If small endian first byte of version should be > 'x\00'
            if buff[4:5] == b'\x00':
                big_endian = True

        # Indicate bigendian in format
        if big_endian:
            self.opts[0][2] = '>' + self.opts[0][2]

        # Apply and parse the default header
        super().__init__(tuple(self.opts))
        super().unpack(buff[0:36])

        if self['StrucLength'] < 0:
            raise PYIFError('RFH2 - "StrucLength" is negative. Check numeric encoding.')

        if len(buff) > 36:
            if self['StrucLength'] > len(buff):
                raise PYIFError('RFH2 - Buffer too short. Expected: %s Buffer Length: %s'
                                % (self['StrucLength'], len(buff)))

        # Extract only the string containing the xml folders and loop
        s = buff[36:self['StrucLength']]

        while s:
            # First 4 bytes is the folder length. supposed to divide by 4.
            len_bytes = s[0:4]
            if big_endian:
                folder_length = struct.unpack('>l', len_bytes)[0]
            else:
                folder_length = struct.unpack('<l', len_bytes)[0]

            # Move on past four byte length
            s = s[4:]

            # Extract the folder string
            folder_data = s[:folder_length]

            # Check that the folder is valid xml and get the root tag name.
            try:
                folder_name = ET.fromstring(folder_data).tag
            except Exception as e:
                raise PYIFError('RFH2 - XML Folder not well formed. Exception: %s' % str(e)) from e

            # Append folder length and folder string to self.opts types
            self.opts.append([folder_name + 'Length', (folder_length), MQLONG_TYPE])
            self.opts.append([folder_name, folder_data, '%is' % folder_length])
            # Move on past the folder
            s = s[folder_length:]

        # Save the current values
        saved_values = self.get()

        # Apply the new opts
        super().__init__(tuple(self.opts))

        # Set from saved values
        self.set(**saved_values)

    def get_folders(self):
        """Return the list of folders in this RFH2"""
        folders = []
        d = self.get()
        for f in d:
            # All the folders have a pair of attributes: xxx and xxxLength
            if f.endswith("Length") and f != "StrucLength":
                f = f[0:len(f) - 6]
                folders.append(f)
        return folders



"""Define the exception classes corresponding to MQI errors and errors
within this library.
"""

# Copyright (c) 2025 IBM Corporation and other Contributors. All Rights Reserved.
# Copyright (c) 2009-2024 Dariusz Suchojad. All Rights Reserved.

try:
    from typing import Any, Dict
except ImportError:
    pass

from ibmmq import CMQC, CMQSTRC

class Error(Exception):
    """ Base class for all exceptions from this library.
    """

# ################################################################################################################################

class MQMIError(Error):
    """ Exception class for MQI level errors: hold the MQRC and MQCC values
    """
    errStringDicts = (CMQSTRC.MQRC_DICT, CMQSTRC.MQRCCF_DICT)
    comp = CMQC.MQCC_OK
    reason = CMQC.MQRC_NONE

    def __init__(self, comp: int, reason: int, **kw: Dict[str, Any]):
        """ Construct the error object with MQI completion code 'comp' and reason code 'reason'.
        """
        self.comp, self.reason = comp, reason

        for k, v in kw.items():
            setattr(self, k, v)

    def __str__(self) -> str:
        return 'MQI Error. Comp %d, Reason %d: %s' % (self.comp, self.reason, self.error_as_string())

    def error_as_string(self) -> str:
        """ Return the exception object MQI warning/failed reason as its mnemonic string.
        """
        if self.comp == CMQC.MQCC_OK:
            return 'OK'

        if self.comp == CMQC.MQCC_WARNING:
            pfx = 'WARNING: '
        else:
            pfx = 'FAILED: '

        for d in MQMIError.errStringDicts:
            if self.reason in d:
                return pfx + d[self.reason]
        return pfx + 'Error code ' + str(self.reason) + ' not defined'

    errorAsString = error_as_string

# ################################################################################################################################

class PYIFError(Error):
    """ Exception class for errors generated by this library
    """
    def __init__(self, e: str):
        self.error = e

    def __str__(self) -> str:
        return 'PYIBMMQ Error: ' + str(self.error)



"""MQSTS: Status returned  by MQSTAT"""

# Copyright (c) 2025 IBM Corporation and other Contributors. All Rights Reserved.
# Copyright (c) 2009-2024 Dariusz Suchojad. All Rights Reserved.

from mqcommon import *
from mqopts import MQOpts
from ibmmq import CMQC, ibmmqc

class STS(MQOpts):
    """ Construct an MQSTS Structure with default values as per MQI.
    The default values may be overridden by the optional keyword arguments 'kw'.
    """
    def __init__(self, **kw):
        opts = [['_StrucId', CMQC.MQSTS_STRUC_ID, '4s'],
                ['Version', CMQC.MQSTS_VERSION_2, MQLONG_TYPE],
                ['CompCode', CMQC.MQCC_OK, MQLONG_TYPE],
                ['Reason', CMQC.MQRC_NONE, MQLONG_TYPE],
                ['PutSuccessCount', 0, MQLONG_TYPE],
                ['PutWarningCount', 0, MQLONG_TYPE],
                ['PutFailureCount', 0, MQLONG_TYPE],
                ['ObjectType', CMQC.MQOT_Q, MQLONG_TYPE],
                ['ObjectName', b'', '48s'],
                ['ObjectQMgrName', b'', '48s'],
                ['ResolvedObjectName', b'', '48s'],
                ['ResolvedQMgrName', b'', '48s']]

        sts_current_version = ibmmqc.__strucversions__.get("sts", 1)
        if sts_current_version >= CMQC.MQSTS_VERSION_2:
            opts += [
                # ObjectString
                ['ObjectStringVSPtr', 0, 'P'],
                ['ObjectStringVSOffset', (0), MQLONG_TYPE],
                ['ObjectStringVSBufSize', (0), MQLONG_TYPE],
                ['ObjectStringVSLength', (0), MQLONG_TYPE],
                ['ObjectStringVSCCSID', (0), MQLONG_TYPE],

                # SubName
                ['SubNameVSPtr', 0, 'P'],
                ['SubNameVSOffset', (0), MQLONG_TYPE],
                ['SubNameVSBufSize', (0), MQLONG_TYPE],
                ['SubNameVSLength', (0), MQLONG_TYPE],
                ['SubNameVSCCSID', (0), MQLONG_TYPE],

                ['OpenOptions', 0, MQLONG_TYPE],
                ['SubOptions', 0, MQLONG_TYPE]]

        super().__init__(tuple(opts), **kw)



# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example shows a simple execution of a PCF command to display connections
and extract the program names. It uses the ByteString constructor for PCF elements.

The "admin" credentials are used to avoid needing to grant application users
additional privileges. We use the CSP class to provide those credentials.
"""

import ibmmq as mq

no_queues = False
queue_manager = 'QM1'
channel = 'DEV.ADMIN.SVRCONN'
host = '127.0.0.1'
port = '1414'

conn_info = '%s(%s)' % (host, port)

csp = mq.CSP()
csp.CSPUserId = 'admin'
csp.CSPPassword = 'password'


# The parameters needed for the INQUIRE CONNECTIONS command
argsName = {mq.CMQCFC.MQBACF_GENERIC_CONNECTION_ID: mq.ByteString('')}


qmgr = mq.connect(queue_manager, channel, conn_info, csp=csp)
pcf = mq.PCFExecute(qmgr, response_wait_interval=15000)

print()
print("Listing connections...")
try:
    response = pcf.MQCMD_INQUIRE_CONNECTION(argsName)
except mq.MQMIError as e:
    if e.comp == mq.CMQC.MQCC_FAILED and e.reason == mq.CMQCFC.MQRCCF_CONNECTION_ID_ERROR:
        no_conns = True
        print('No connections matched.')  # Should never happen!
    else:
        raise
else:
    for conn_info in response:
        try:
            appl_tag = conn_info[mq.CMQCFC.MQCACF_APPL_TAG]
            # Convert the byte-form of the app name to a Python string.
            s_appl_tag = mq.to_string(appl_tag)
            print(f'Found application \"{s_appl_tag}\"')
            if s_appl_tag.startswith("python"):
                stashed_conn_tag = conn_info[mq.CMQCFC.MQBACF_CONN_TAG]
        except UnicodeError:
            print(f'Failure to decode msg: {appl_tag}')
        except KeyError:
            print('Cannot find queue name in response: ', conn_info)

qmgr.disconnect()

print("Done.")



"""MQCB/MQCTL implementation, including the proxy Python callback"""

# Copyright (c) 2025,2026 IBM Corporation and other Contributors. All Rights Reserved.
# Copyright (c) 2009-2024 Dariusz Suchojad. All Rights Reserved.

from threading import Lock

from mqcommon import *
from mqerrors import *
from ibmmq import CMQC, GMO, MD, CBC, CBD, ibmmqc
import mqlog

# pylint: disable=no-member

_callback_lock = Lock()

# A couple of classes hold data that's given to the callback function
# _cbCBD has a per-callback block; _cbCTLO has a per-connection block
class _cbCBD:
    def __init__(self, obj, cbd, otel_options):
        self.callback_function = cbd.CallbackFunction
        self.callback_area = cbd.CallbackArea
        self.object = obj
        self.otel_options = otel_options


class _cbCTLO:
    def __init__(self, ctlo):
        self.connection_area = ctlo.ConnectionArea


_stashedCBD: dict[str, _cbCBD] = {}
_stashedCTLO: dict[int, _cbCTLO] = {}

# Create the key that's going to give us stashed info for a queue or qmgr
def _make_key(hc, ho):
    s = str(hc) + "/" + str(ho)
    return s

# Create a key to cover all stashed info for a qmgr connection
def _make_partial_key(hc):
    s = str(hc) + "/"
    return s

# The hConn and hObj are the actual numbers, while we also stash the object (Queue or QueueManager) and other fields
# from the CBD structure
def _save_callback(obj, qmgr, queue, cbd, otel_opts):
    key = _make_key(qmgr, queue)
    with _callback_lock:
        _stashedCBD[key] = _cbCBD(obj, cbd, otel_opts)

def _delete_callback(hconn, hobj):
    key = _make_key(hconn, hobj)
    try:
        with _callback_lock:
            del _stashedCBD[key]
    except KeyError:
        pass

def _delete_all_callbacks(hconn):
    pk = _make_partial_key(hconn)
    with _callback_lock:
        for key in list(_stashedCBD.keys()):
            if key.startswith(pk):
                try:
                    del _stashedCBD[key]
                except KeyError:
                    pass
        key = str(hconn)
        try:
            del _stashedCTLO[key]
        except KeyError:
            pass

def _save_connection_area(hconn, ctlo):
    with _callback_lock:
        _stashedCTLO[hconn] = _cbCTLO(ctlo)

# This is the Python "proxy" callback invoked from the C proxy.
# Most of the parameters to this function are byte buffers.
#
# It locates the real user-specified callback function, using
# the hConn and hObj. It then calls the user function
# with suitably-reformatted parameters.
def _internal_cb(hc, md, gmo, buf, cbc):
    mqlog.trace_entry("callback:_internal_cb")

    key = _make_key(hc, CBC().unpack(cbc).Hobj)
    try:
        cb = _stashedCBD[key]
    except KeyError as exc:
        # Should not happen as we've got control of the map. But just in case ...
        mqlog.trace_exit("callback:_internal_cb", ep=1)
        raise KeyError(f'Cannot find key {key} in callback map') from exc

    try:
        qmgr = cb.object.get_queue_manager()
        queue = cb.object
    except AttributeError:
        qmgr = cb.object
        queue = None

    cbc_up = CBC().unpack(cbc)
    cbc_up.CallbackArea = cb.callback_area
    cbc_up.CallbackFunction = cb.callback_function
    cbc_up.ConnectionArea = 0
    try:
        ca = _stashedCTLO[hc]
        cbc_up.ConnectionArea = ca.connection_area
    except KeyError:
        pass

    # There's a difference between local and client connections
    # when CallbackType=EVENT_CALL: in clients, the MD may be None. Whereas
    # local bindings always gives an MD.
    # So we fake a dummy MD (and similar for GMO) even if it's not going to be used, so
    # both modes look the same.
    if md:
        md_up = MD().unpack(md)
    else:
        md_up = MD()

    if gmo:
        gmo_up = GMO().unpack(gmo)
    else:
        gmo_up = GMO()

    removed = 0
    if OTelFunctions.get_trace_after:
        removed = OTelFunctions.get_trace_after(queue, gmo_up, md_up, cb.otel_options, buf, True)

    # Call the real user function with the unpacked forms of the structures.
    # If the callback_function is actually a method within a class, then the "self"
    # parameter is automatically added to the parameters.
    if buf:
        msg = buf[removed:]
    else:
        msg = buf
    mqlog.trace("before calling user callback function")
    cb.callback_function(queue_manager=qmgr, queue=queue, md=md_up, gmo=gmo_up, msg=msg, cbc=cbc_up)
    mqlog.trace("after  calling user callback function")

    mqlog.trace_exit("callback:_internal_cb")

def real_cb(obj, kwargs):
    """
    Register or deregister a user function to be called when a message arrives or an event occurs.
    The function and relevant correlator blocks are stashed locally, so they can be restored
    in the proxy callback function.
    The related MQCTL is called as a method on the QMgr object
    """
    mqlog.trace_entry("callback:real_cb")

    operation = kwargs['operation'] if 'operation' in kwargs else CMQC.MQOP_REGISTER
    md = kwargs['md'] if 'md' in kwargs else MD()
    cbd = kwargs['cbd'] if 'cbd' in kwargs else CBD()
    gmo = kwargs['gmo'] if 'gmo' in kwargs else GMO()
    otel_options = kwargs['otel_options'] if 'otel_options' in kwargs else None

    if not isinstance(cbd, CBD):
        mqlog.trace_exit("callback:real_cb", ep=1)
        raise TypeError("cbd must be an instance of CBD")
    if not isinstance(md, MD):
        mqlog.trace_exit("callback:real_cb", ep=2)
        raise TypeError("md must be an instance of MD")
    if not isinstance(gmo, GMO):
        mqlog.trace_exit("callback:real_cb", ep=3)
        raise TypeError("gmo must be an instance of GMO")
    if otel_options and not isinstance(otel_options, OTelOptions):
        mqlog.trace_exit("callback:real_cb", ep=4)
        raise TypeError("otel_options must be an instance of OTelOptions")

    # The object must be either a queue or qmgr class.
    # But we can't use isinstance here because of circular
    # import nesting. So try to treat it as a Queue and if
    # that fails, treat it as a QMgr (without a 'cast')
    try:
        hconn = obj.get_queue_manager().get_handle()
        hobj = obj.get_handle()
    except AttributeError:
        hconn = obj.get_handle()
        hobj = CMQC.MQHO_NONE

    # These fields do not need to be passed to the
    # C layer as they point at Python objects and get stashed locally. Instead,
    # set them to 0 so as to allow pack() to work. Then restore them after
    # the MQCB call, to allow the stashing and so the app doesn't
    # see the change.
    original_cbf = cbd.CallbackFunction
    original_cba = cbd.CallbackArea
    cbd.CallbackFunction = 0
    cbd.CallbackArea = 0

    if OTelFunctions.get_trace_before:
        OTelFunctions.get_trace_before(hconn, hobj, gmo, True)

    rv = ibmmqc.MQCB(hconn, operation, cbd.pack(), hobj, md.pack(), gmo.pack())

    # Restore the fields so they can be stashed in the _save_callback function
    cbd.CallbackFunction = original_cbf
    cbd.CallbackArea = original_cba

    if rv[0]:
        mqlog.trace_exit("callback:real_cb", ep=5, rc=rv[-1])
        raise MQMIError(rv[-2], rv[-1])

    if operation == CMQC.MQOP_REGISTER:
        _save_callback(obj, hconn, hobj, cbd, otel_options)
    elif operation == CMQC.MQOP_DEREGISTER:
        _delete_callback(hconn, hobj)
    mqlog.trace_exit("callback:real_cb")


# Initialise the C layer with the address of this module's proxy callback function.
ibmmqc.MQCBINIT(_internal_cb)



# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example shows how to use the MQINQ and MQSET operations
"""

from datetime import datetime
import random
import ibmmq as mq
from ibmmq import CMQC, CMQSTRC

def dump_attrs(attributes):
    """ Dump the map of attributes and their values."""
    print("Object attributes are: ")
    for k in attributes:
        try:
            k_desc = attr_dicts[k]
        except KeyError:
            k_desc = "Unknown"

        print(f"  {k} [{k_desc}] = {attributes[k]}")


# Merge all the attribute strings into a single dict
attr_dicts = {}
for d in (CMQSTRC.MQIA_DICT, CMQSTRC.MQIACF_DICT, CMQSTRC.MQIACH_DICT,
          CMQSTRC.MQCA_DICT, CMQSTRC.MQCACF_DICT, CMQSTRC.MQCACH_DICT):
    attr_dicts.update(d)

queue_manager = 'QM1'
channel = 'DEV.APP.SVRCONN'
host = '127.0.0.1'
port = '1414'
conn_info = '%s(%s)' % (host, port)
user = 'app'
password = 'password'

csp = mq.CSP()
csp.CSPUserId = user
csp.CSPPassword = password

queue_name = "DEV.QUEUE.1"

# Connect to qmgr
qmgr = mq.connect(queue_manager, channel, conn_info, csp=csp)

# This shows the original single-attribute version of MQINQ
# Note that we haven't explicitly opened the qmgr for INQUIRE
print("INQ (1) on QMgr")
print("QMgr version: ", qmgr.inquire(CMQC.MQCA_VERSION))

# And using the multi-selector model
print()
print("INQ (2) on QMgr")

selectors = [CMQC.MQCA_VERSION,
             CMQC.MQIA_COMMAND_LEVEL]
attrs = qmgr.inquire(selectors)
dump_attrs(attrs)

# Get access to the queue for INQ and SET
od = mq.OD()
od.ObjectName = queue_name
q = mq.Queue(qmgr, od, CMQC.MQOO_SET | CMQC.MQOO_INQUIRE)

# Doing an MQSET for the queue, to change two attributes in a single operation.
# Use a random number for trigger depth so we can see it change with repeated executions.
print()
print("SET on Queue")
now = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
selectors = {CMQC.MQCA_TRIGGER_DATA: now,
             CMQC.MQIA_TRIGGER_DEPTH: random.randint(1, 99)}

# print("Set selectors: ", selectors)
dump_attrs(selectors)
q.set(selectors)

# Doing an MQINQ for a queue, using the multi-selector approach. This should show
# values including those we just did an MQSET for.
selectors = [CMQC.MQCA_Q_DESC,
             CMQC.MQIA_TRIGGER_DEPTH,
             CMQC.MQCA_BACKOUT_REQ_Q_NAME,
             CMQC.MQCA_TRIGGER_DATA,
             CMQC.MQIA_CURRENT_Q_DEPTH]
attrs = q.inquire(selectors)

print()
print("INQ on Queue")
dump_attrs(attrs)

q.close()
qmgr.disconnect()

print()
print("Done.")



# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example shows a simple execution of a PCF command to display channels matching a pattern. The PCF
parameters are passed as a dictionary of attribute name and value pairs. Compare with the style used in the
dis_queues example.

The "admin" credentials are used to avoid needing to grant application users
additional privileges.
"""

import logging

import ibmmq as mq

logging.basicConfig(level=logging.INFO)

queue_manager = 'QM1'
channel = 'DEV.ADMIN.SVRCONN'
host = '127.0.0.1'
port = '1414'
conn_info = '%s(%s)' % (host, port)
user = 'admin'
password = 'password'

prefix = 'SYSTEM.*'

args = {mq.CMQCFC.MQCACH_CHANNEL_NAME: prefix}

qmgr = mq.connect(queue_manager, channel, conn_info, user, password)
pcf = mq.PCFExecute(qmgr)

try:
    response = pcf.MQCMD_INQUIRE_CHANNEL(args)
except mq.MQMIError as e:
    if e.comp == mq.CMQC.MQCC_FAILED and e.reason == mq.CMQC.MQRC_UNKNOWN_OBJECT_NAME:
        logging.info('No channels matched prefix `%s`', prefix)
    else:
        raise
else:
    for channel_info in response:
        channel_name = channel_info[mq.CMQCFC.MQCACH_CHANNEL_NAME]
        logging.info('Found channel `%s`', channel_name)

qmgr.disconnect()

print("Done.")



import sys
import os
import platform

sys.path.insert(0,os.path.dirname(__file__))

_plat=platform.system()
_mach=platform.machine()

# Load a different variant of CMQC for each platform
# The other definition files do not vary by platform.
# If the OS and/or hardware are unrecognised, we end up
# loading the Linux X64 definitions.
if _plat == "Linux":
    if _mach == "x86_64":
        from _CMQC_linux_x64 import *
    elif _mach == "aarch64":
        from _CMQC_linux_arm64 import *
    elif _mach == "s390x":
        from _CMQC_linux_s390x import *
    elif _mach == "ppc64le" or _mach == "ppcle":
        from _CMQC_linux_ppcle import *
    else:
        from _CMQC_linux_x64 import *
elif _plat == "AIX":
    from _CMQC_aix import *
elif _plat == "Darwin":
    from _CMQC_macos import *
elif _plat == "Windows":
    from _CMQC_windows import *
else:
    from _CMQC_linux_x64 import *




# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example generates an error when connecting. It shows both how the exception
can be caught and inspected, and how the values can be converted to readable text.
"""

import ibmmq as mq
from ibmmq import CMQC, CMQSTRC

queue_manager = 'QM1'
channel = 'DEV.APP.SVRCONN'
host = 'localhost.invalid'  # Note the hostname that should never be resolvable
port = '1414'
conn_info = '%s(%s)' % (host, port)
user = 'app'
password = 'password'

try:
    qmgr = mq.connect(queue_manager, channel, conn_info, user, password)
except mq.MQMIError as e:
    if e.comp == CMQC.MQCC_FAILED and e.reason == CMQC.MQRC_HOST_NOT_AVAILABLE:
        print(f'Host `{host}` does not exist.')
    print(f"MQI Error is CC:{e.comp} [{CMQSTRC.MQCC_DICT[e.comp]}] RC:{e.reason} [{CMQSTRC.MQRC_DICT[e.reason]}]")



"""
----------------------------------------------------------------

                     IBM MQ                                  

   FILE NAME:      CMQXC                                     

   DESCRIPTION:    Declarations for Exits and MQCD           

----------------------------------------------------------------

   <copyright
   notice="lm-source-program"
   pids="5724-H72"
   years="1993,2026"
   crc="0" >

   Licensed Materials - Property of IBM

   5724-H72

   (C) Copyright IBM Corp. 1993, 2026 All Rights Reserved.

   US Government Users Restricted Rights - Use, duplication or
   disclosure restricted by GSA ADP Schedule Contract with
   IBM Corp.
   </copyright>
----------------------------------------------------------------

   FUNCTION:       This file declares the functions,           
                   structures and named constants for exits    
                   and MQCD.                                   

   PROCESSOR:      Python

----------------------------------------------------------------
<BEGIN_BUILDINFO>
   Generated on:  1/20/26, 9:18 AM                              
   Build Level:   p945-L260120                                  
   Build Type:    Production                                    
   Pointer Size:  64 Bit                                        
<END_BUILDINFO>
----------------------------------------------------------------
"""

#----------------------------------------------------------------#
# Values Related to MQCD Structure                               #
#----------------------------------------------------------------#

# Structure Version Number
MQCD_VERSION_1                           = 1
MQCD_VERSION_2                           = 2
MQCD_VERSION_3                           = 3
MQCD_VERSION_4                           = 4
MQCD_VERSION_5                           = 5
MQCD_VERSION_6                           = 6
MQCD_VERSION_7                           = 7
MQCD_VERSION_8                           = 8
MQCD_VERSION_9                           = 9
MQCD_VERSION_10                          = 10
MQCD_VERSION_11                          = 11
MQCD_VERSION_12                          = 12
MQCD_CURRENT_VERSION                     = 12

# Structure Length
MQCD_LENGTH_1                            = 984
MQCD_LENGTH_2                            = 1312
MQCD_LENGTH_3                            = 1480
MQCD_LENGTH_4                            = 1568
MQCD_LENGTH_5                            = 1584
MQCD_LENGTH_6                            = 1688
MQCD_LENGTH_7                            = 1792
MQCD_LENGTH_8                            = 1888
MQCD_LENGTH_9                            = 1912
MQCD_LENGTH_10                           = 1920
MQCD_LENGTH_11                           = 1984
MQCD_LENGTH_12                           = 1992
MQCD_CURRENT_LENGTH                      = 1992

# Channel Types
MQCHT_SENDER                             = 1
MQCHT_SERVER                             = 2
MQCHT_RECEIVER                           = 3
MQCHT_REQUESTER                          = 4
MQCHT_ALL                                = 5
MQCHT_CLNTCONN                           = 6
MQCHT_SVRCONN                            = 7
MQCHT_CLUSRCVR                           = 8
MQCHT_CLUSSDR                            = 9
MQCHT_MQTT                               = 10
MQCHT_AMQP                               = 11

# Channel Compression
MQCOMPRESS_NOT_AVAILABLE                 = (-1)
MQCOMPRESS_NONE                          = 0
MQCOMPRESS_RLE                           = 1
MQCOMPRESS_ZLIBFAST                      = 2
MQCOMPRESS_ZLIBHIGH                      = 4
MQCOMPRESS_SYSTEM                        = 8
MQCOMPRESS_LZ4FAST                       = 16
MQCOMPRESS_LZ4HIGH                       = 32
MQCOMPRESS_ANY                           = 0x0fffffff

# Transport Types
MQXPT_ALL                                = (-1)
MQXPT_LOCAL                              = 0
MQXPT_LU62                               = 1
MQXPT_TCP                                = 2
MQXPT_NETBIOS                            = 3
MQXPT_SPX                                = 4
MQXPT_DECNET                             = 5
MQXPT_UDP                                = 6

# Put Authority
MQPA_DEFAULT                             = 1
MQPA_CONTEXT                             = 2
MQPA_ONLY_MCA                            = 3
MQPA_ALTERNATE_OR_MCA                    = 4

# Channel Data Conversion
MQCDC_SENDER_CONVERSION                  = 1
MQCDC_NO_SENDER_CONVERSION               = 0

# MCA Types
MQMCAT_PROCESS                           = 1
MQMCAT_THREAD                            = 2

# NonPersistent-Message Speeds
MQNPMS_NORMAL                            = 1
MQNPMS_FAST                              = 2

# SSL Client Authentication
MQSCA_REQUIRED                           = 0
MQSCA_OPTIONAL                           = 1
MQSCA_NEVER_REQUIRED                     = 2

# KeepAlive Interval
MQKAI_AUTO                               = (-1)

# Connection Affinity Values
MQCAFTY_NONE                             = 0
MQCAFTY_PREFERRED                        = 1

# Client Reconnect
MQRCN_NO                                 = 0
MQRCN_YES                                = 1
MQRCN_Q_MGR                              = 2
MQRCN_DISABLED                           = 3

# Protocol
MQPROTO_MQTTV3                           = 1
MQPROTO_HTTP                             = 2
MQPROTO_AMQP                             = 3
MQPROTO_MQTTV311                         = 4

# Security Protocol
MQSECPROT_NONE                           = 0
MQSECPROT_SSLV30                         = 1
MQSECPROT_TLSV10                         = 2
MQSECPROT_TLSV12                         = 4
MQSECPROT_TLSV13                         = 8

# SPL Protection
MQSPL_PASSTHRU                           = 0
MQSPL_REMOVE                             = 1
MQSPL_AS_POLICY                          = 2

#----------------------------------------------------------------#
# Values Related to MQACH Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQACH_STRUC_ID                           = b"ACH "

# Structure Identifier (array form)
MQACH_STRUC_ID_ARRAY                     = [b"A", b"C", b"H", b" "]

# Structure Version Number
MQACH_VERSION_1                          = 1
MQACH_CURRENT_VERSION                    = 1

# Structure Length
MQACH_LENGTH_1                           = 72
MQACH_CURRENT_LENGTH                     = 72

#----------------------------------------------------------------#
# Values Related to MQAXC Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQAXC_STRUC_ID                           = b"AXC "

# Structure Identifier (array form)
MQAXC_STRUC_ID_ARRAY                     = [b"A", b"X", b"C", b" "]

# Structure Version Number
MQAXC_VERSION_1                          = 1
MQAXC_VERSION_2                          = 2
MQAXC_CURRENT_VERSION                    = 2

# Structure Length
MQAXC_LENGTH_1                           = 392
MQAXC_LENGTH_2                           = 424
MQAXC_CURRENT_LENGTH                     = 424

# Environments
MQXE_OTHER                               = 0
MQXE_MCA                                 = 1
MQXE_MCA_SVRCONN                         = 2
MQXE_COMMAND_SERVER                      = 3
MQXE_MQSC                                = 4
MQXE_MCA_CLNTCONN                        = 5

#----------------------------------------------------------------#
# Values Related to MQAXP Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQAXP_STRUC_ID                           = b"AXP "

# Structure Identifier (array form)
MQAXP_STRUC_ID_ARRAY                     = [b"A", b"X", b"P", b" "]

# Structure Version Number
MQAXP_VERSION_1                          = 1
MQAXP_VERSION_2                          = 2
MQAXP_CURRENT_VERSION                    = 2

# Structure Length
MQAXP_LENGTH_1                           = 256
MQAXP_CURRENT_LENGTH                     = 256

# API Caller Types
MQXACT_EXTERNAL                          = 1
MQXACT_INTERNAL                          = 2

# Problem Determination Area
MQXPDA_NONE                              = b"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"\
                                           b"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"\
                                           b"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"

# Problem Determination Area (array form)
MQXPDA_NONE_ARRAY                        = [b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0'
                                          , b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0'
                                          , b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0'
                                          , b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0'
                                          , b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0'
                                          , b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0']

# API Function Identifiers
MQXF_INIT                                = 1
MQXF_TERM                                = 2
MQXF_CONN                                = 3
MQXF_CONNX                               = 4
MQXF_DISC                                = 5
MQXF_OPEN                                = 6
MQXF_CLOSE                               = 7
MQXF_PUT1                                = 8
MQXF_PUT                                 = 9
MQXF_GET                                 = 10
MQXF_DATA_CONV_ON_GET                    = 11
MQXF_INQ                                 = 12
MQXF_SET                                 = 13
MQXF_BEGIN                               = 14
MQXF_CMIT                                = 15
MQXF_BACK                                = 16
MQXF_STAT                                = 18
MQXF_CB                                  = 19
MQXF_CTL                                 = 20
MQXF_CALLBACK                            = 21
MQXF_SUB                                 = 22
MQXF_SUBRQ                               = 23
MQXF_XACLOSE                             = 24
MQXF_XACOMMIT                            = 25
MQXF_XACOMPLETE                          = 26
MQXF_XAEND                               = 27
MQXF_XAFORGET                            = 28
MQXF_XAOPEN                              = 29
MQXF_XAPREPARE                           = 30
MQXF_XARECOVER                           = 31
MQXF_XAROLLBACK                          = 32
MQXF_XASTART                             = 33
MQXF_AXREG                               = 34
MQXF_AXUNREG                             = 35

#----------------------------------------------------------------#
# Values Related to MQCDC Structure                              #
#----------------------------------------------------------------#

# Structure Version Number
MQCDC_VERSION_1                          = 1
MQCDC_VERSION_2                          = 2
MQCDC_VERSION_3                          = 3
MQCDC_VERSION_4                          = 4
MQCDC_VERSION_5                          = 5
MQCDC_VERSION_6                          = 6
MQCDC_VERSION_7                          = 7
MQCDC_VERSION_8                          = 8
MQCDC_VERSION_9                          = 9
MQCDC_VERSION_10                         = 10
MQCDC_VERSION_11                         = 11
MQCDC_VERSION_12                         = 12
MQCDC_CURRENT_VERSION                    = 12

# Structure Length
MQCDC_LENGTH_1                           = 984
MQCDC_LENGTH_2                           = 1312
MQCDC_LENGTH_3                           = 1480
MQCDC_LENGTH_4                           = 1568
MQCDC_LENGTH_5                           = 1584
MQCDC_LENGTH_6                           = 1688
MQCDC_LENGTH_7                           = 1792
MQCDC_LENGTH_8                           = 1888
MQCDC_LENGTH_9                           = 1912
MQCDC_LENGTH_10                          = 1920
MQCDC_LENGTH_11                          = 1984
MQCDC_LENGTH_12                          = 1992
MQCDC_CURRENT_LENGTH                     = 1992

#----------------------------------------------------------------#
# Values Related to MQCXP Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQCXP_STRUC_ID                           = b"CXP "

# Structure Identifier (array form)
MQCXP_STRUC_ID_ARRAY                     = [b"C", b"X", b"P", b" "]

# Structure Version Number
MQCXP_VERSION_1                          = 1
MQCXP_VERSION_2                          = 2
MQCXP_VERSION_3                          = 3
MQCXP_VERSION_4                          = 4
MQCXP_VERSION_5                          = 5
MQCXP_VERSION_6                          = 6
MQCXP_VERSION_7                          = 7
MQCXP_VERSION_8                          = 8
MQCXP_VERSION_9                          = 9
MQCXP_CURRENT_VERSION                    = 9

# Structure Length
MQCXP_LENGTH_3                           = 156
MQCXP_LENGTH_4                           = 156
MQCXP_LENGTH_5                           = 160
MQCXP_LENGTH_6                           = 200
MQCXP_LENGTH_7                           = 208
MQCXP_LENGTH_8                           = 224
MQCXP_LENGTH_9                           = 240
MQCXP_CURRENT_LENGTH                     = 240

# Exit Response 2
MQXR2_PUT_WITH_DEF_ACTION                = 0
MQXR2_PUT_WITH_DEF_USERID                = 1
MQXR2_PUT_WITH_MSG_USERID                = 2
MQXR2_USE_AGENT_BUFFER                   = 0
MQXR2_USE_EXIT_BUFFER                    = 4
MQXR2_DEFAULT_CONTINUATION               = 0
MQXR2_CONTINUE_CHAIN                     = 8
MQXR2_SUPPRESS_CHAIN                     = 16
MQXR2_STATIC_CACHE                       = 0
MQXR2_DYNAMIC_CACHE                      = 32

# Capability Flags
MQCF_NONE                                = 0x00000000
MQCF_DIST_LISTS                          = 0x00000001

#----------------------------------------------------------------#
# Values Related to MQDXP Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQDXP_STRUC_ID                           = b"DXP "

# Structure Identifier (array form)
MQDXP_STRUC_ID_ARRAY                     = [b"D", b"X", b"P", b" "]

# Structure Version Number
MQDXP_VERSION_1                          = 1
MQDXP_VERSION_2                          = 2
MQDXP_CURRENT_VERSION                    = 2

# Structure Length
MQDXP_LENGTH_1                           = 44
MQDXP_LENGTH_2                           = 56
MQDXP_CURRENT_LENGTH                     = 56

# Exit Response
MQXDR_OK                                 = 0
MQXDR_CONVERSION_FAILED                  = 1

#----------------------------------------------------------------#
# Values Related to MQNXP Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQNXP_STRUC_ID                           = b"NXP "

# Structure Identifier (array form)
MQNXP_STRUC_ID_ARRAY                     = [b"N", b"X", b"P", b" "]

# Structure Version Number
MQNXP_VERSION_1                          = 1
MQNXP_VERSION_2                          = 2
MQNXP_CURRENT_VERSION                    = 2

# Structure Length
MQNXP_LENGTH_1                           = 64
MQNXP_LENGTH_2                           = 72
MQNXP_CURRENT_LENGTH                     = 72

#----------------------------------------------------------------#
# Values Related to MQPBC Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQPBC_STRUC_ID                           = b"PBC "

# Structure Identifier (array form)
MQPBC_STRUC_ID_ARRAY                     = [b"P", b"B", b"C", b" "]

# Structure Version Number
MQPBC_VERSION_1                          = 1
MQPBC_VERSION_2                          = 2
MQPBC_CURRENT_VERSION                    = 2

# Structure Length
MQPBC_LENGTH_1                           = 32
MQPBC_LENGTH_2                           = 40
MQPBC_CURRENT_LENGTH                     = 40

#----------------------------------------------------------------#
# Values Related to MQPSXP Structure                             #
#----------------------------------------------------------------#

# Structure Identifier
MQPSXP_STRUC_ID                          = b"PSXP"

# Structure Identifier (array form)
MQPSXP_STRUC_ID_ARRAY                    = [b"P", b"S", b"X", b"P"]

# Structure Version Number
MQPSXP_VERSION_1                         = 1
MQPSXP_VERSION_2                         = 2
MQPSXP_CURRENT_VERSION                   = 2

# Structure Length
MQPSXP_LENGTH_1                          = 176
MQPSXP_LENGTH_2                          = 184
MQPSXP_CURRENT_LENGTH                    = 184

#----------------------------------------------------------------#
# Values Related to MQSBC Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQSBC_STRUC_ID                           = b"SBC "

# Structure Identifier (array form)
MQSBC_STRUC_ID_ARRAY                     = [b"S", b"B", b"C", b" "]

# Structure Version Number
MQSBC_VERSION_1                          = 1
MQSBC_CURRENT_VERSION                    = 1

# Structure Length
MQSBC_LENGTH_1                           = 288
MQSBC_CURRENT_LENGTH                     = 288

#----------------------------------------------------------------#
# Values Related to MQWDR Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQWDR_STRUC_ID                           = b"WDR "

# Structure Identifier (array form)
MQWDR_STRUC_ID_ARRAY                     = [b"W", b"D", b"R", b" "]

# Structure Version Number
MQWDR_VERSION_1                          = 1
MQWDR_VERSION_2                          = 2
MQWDR_CURRENT_VERSION                    = 2

# Structure Length
MQWDR_LENGTH_1                           = 124
MQWDR_LENGTH_2                           = 136
MQWDR_CURRENT_LENGTH                     = 136

# Queue Manager Flags
MQQMF_REPOSITORY_Q_MGR                   = 0x00000002
MQQMF_CLUSSDR_USER_DEFINED               = 0x00000008
MQQMF_CLUSSDR_AUTO_DEFINED               = 0x00000010
MQQMF_AVAILABLE                          = 0x00000020

#----------------------------------------------------------------#
# Values Related to MQWDR Structure                              #
#----------------------------------------------------------------#

# Structure Length
MQWDR1_LENGTH_1                          = 124
MQWDR1_CURRENT_LENGTH                    = 124

#----------------------------------------------------------------#
# Values Related to MQWDR2 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWDR2_LENGTH_1                          = 124
MQWDR2_LENGTH_2                          = 136
MQWDR2_CURRENT_LENGTH                    = 136

#----------------------------------------------------------------#
# Values Related to MQWQR Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQWQR_STRUC_ID                           = b"WQR "

# Structure Identifier (array form)
MQWQR_STRUC_ID_ARRAY                     = [b"W", b"Q", b"R", b" "]

# Structure Version Number
MQWQR_VERSION_1                          = 1
MQWQR_VERSION_2                          = 2
MQWQR_VERSION_3                          = 3
MQWQR_VERSION_4                          = 4
MQWQR_CURRENT_VERSION                    = 4

# Structure Length
MQWQR_LENGTH_1                           = 200
MQWQR_LENGTH_2                           = 208
MQWQR_LENGTH_3                           = 212
MQWQR_LENGTH_4                           = 216
MQWQR_CURRENT_LENGTH                     = 216

# Queue Flags
MQQF_LOCAL_Q                             = 0x00000001
MQQF_CLWL_USEQ_ANY                       = 0x00000040
MQQF_CLWL_USEQ_LOCAL                     = 0x00000080

#----------------------------------------------------------------#
# Values Related to MQWQR1 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWQR1_LENGTH_1                          = 200
MQWQR1_CURRENT_LENGTH                    = 200

#----------------------------------------------------------------#
# Values Related to MQWQR2 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWQR2_LENGTH_1                          = 200
MQWQR2_LENGTH_2                          = 208
MQWQR2_CURRENT_LENGTH                    = 208

#----------------------------------------------------------------#
# Values Related to MQWQR3 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWQR3_LENGTH_1                          = 200
MQWQR3_LENGTH_2                          = 208
MQWQR3_LENGTH_3                          = 212
MQWQR3_CURRENT_LENGTH                    = 212

#----------------------------------------------------------------#
# Values Related to MQWQR4 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWQR4_LENGTH_1                          = 200
MQWQR4_LENGTH_2                          = 208
MQWQR4_LENGTH_3                          = 212
MQWQR4_LENGTH_4                          = 216
MQWQR4_CURRENT_LENGTH                    = 216

#----------------------------------------------------------------#
# Values Related to MQWXP Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQWXP_STRUC_ID                           = b"WXP "

# Structure Identifier (array form)
MQWXP_STRUC_ID_ARRAY                     = [b"W", b"X", b"P", b" "]

# Structure Version Number
MQWXP_VERSION_1                          = 1
MQWXP_VERSION_2                          = 2
MQWXP_VERSION_3                          = 3
MQWXP_VERSION_4                          = 4
MQWXP_CURRENT_VERSION                    = 4

# Structure Length
MQWXP_LENGTH_1                           = 224
MQWXP_LENGTH_2                           = 240
MQWXP_LENGTH_3                           = 240
MQWXP_LENGTH_4                           = 248
MQWXP_CURRENT_LENGTH                     = 248

# Cluster Workload Flags
MQWXP_PUT_BY_CLUSTER_CHL                 = 0x00000002

#----------------------------------------------------------------#
# Values Related to MQWXP1 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWXP1_LENGTH_1                          = 224
MQWXP1_CURRENT_LENGTH                    = 224

#----------------------------------------------------------------#
# Values Related to MQWXP2 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWXP2_LENGTH_1                          = 224
MQWXP2_LENGTH_2                          = 240
MQWXP2_CURRENT_LENGTH                    = 240

#----------------------------------------------------------------#
# Values Related to MQWXP3 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWXP3_LENGTH_1                          = 224
MQWXP3_LENGTH_2                          = 240
MQWXP3_LENGTH_3                          = 240
MQWXP3_CURRENT_LENGTH                    = 240

#----------------------------------------------------------------#
# Values Related to MQWXP4 Structure                             #
#----------------------------------------------------------------#

# Structure Length
MQWXP4_LENGTH_1                          = 224
MQWXP4_LENGTH_2                          = 240
MQWXP4_LENGTH_3                          = 240
MQWXP4_LENGTH_4                          = 248
MQWXP4_CURRENT_LENGTH                    = 248

#----------------------------------------------------------------#
# Values Related to MQXEPO Structure                             #
#----------------------------------------------------------------#

# Structure Identifier
MQXEPO_STRUC_ID                          = b"XEPO"

# Structure Identifier (array form)
MQXEPO_STRUC_ID_ARRAY                    = [b"X", b"E", b"P", b"O"]

# Structure Version Number
MQXEPO_VERSION_1                         = 1
MQXEPO_CURRENT_VERSION                   = 1

# Structure Length
MQXEPO_LENGTH_1                          = 40
MQXEPO_CURRENT_LENGTH                    = 40

# Exit Options
MQXEPO_NONE                              = 0x00000000

#----------------------------------------------------------------#
# Values Related to MQXP Structure                               #
#----------------------------------------------------------------#

# Structure Identifier
MQXP_STRUC_ID                            = b"XP  "

# Structure Identifier (array form)
MQXP_STRUC_ID_ARRAY                      = [b"X", b"P", b" ", b" "]

# Structure Version Number
MQXP_VERSION_1                           = 1

# Structure Length
MQXP_LENGTH_1                            = 44
MQXP_CURRENT_LENGTH                      = 44

# Exit Commands
MQXC_MQOPEN                              = 1
MQXC_MQCLOSE                             = 2
MQXC_MQGET                               = 3
MQXC_MQPUT                               = 4
MQXC_MQPUT1                              = 5
MQXC_MQINQ                               = 6
MQXC_MQSET                               = 8
MQXC_MQBACK                              = 9
MQXC_MQCMIT                              = 10
MQXC_MQSUB                               = 42
MQXC_MQSUBRQ                             = 43
MQXC_MQCB                                = 44
MQXC_MQCTL                               = 45
MQXC_MQSTAT                              = 46
MQXC_CALLBACK                            = 48

#----------------------------------------------------------------#
# Values Related to MQXWD Structure                              #
#----------------------------------------------------------------#

# Structure Identifier
MQXWD_STRUC_ID                           = b"XWD "

# Structure Identifier (array form)
MQXWD_STRUC_ID_ARRAY                     = [b"X", b"W", b"D", b" "]

# Structure Version Number
MQXWD_VERSION_1                          = 1

# Structure Length
MQXWD_LENGTH_1                           = 24
MQXWD_CURRENT_LENGTH                     = 24

#----------------------------------------------------------------#
# Values Related to MQ_CLUSTER_WORKLOAD_EXIT Function            #
#----------------------------------------------------------------#
# Channel exit

#----------------------------------------------------------------#
# General Values Related to Exits                                #
#----------------------------------------------------------------#
# Exit Identifiers
MQXT_API_CROSSING_EXIT                   = 1
MQXT_API_EXIT                            = 2
MQXT_CHANNEL_SEC_EXIT                    = 11
MQXT_CHANNEL_MSG_EXIT                    = 12
MQXT_CHANNEL_SEND_EXIT                   = 13
MQXT_CHANNEL_RCV_EXIT                    = 14
MQXT_CHANNEL_MSG_RETRY_EXIT              = 15
MQXT_CHANNEL_AUTO_DEF_EXIT               = 16
MQXT_CLUSTER_WORKLOAD_EXIT               = 20
MQXT_PUBSUB_ROUTING_EXIT                 = 21
MQXT_PUBLISH_EXIT                        = 22
MQXT_PRECONNECT_EXIT                     = 23

# Exit Reasons
MQXR_BEFORE                              = 1
MQXR_AFTER                               = 2
MQXR_CONNECTION                          = 3
MQXR_BEFORE_CONVERT                      = 4
MQXR_INIT                                = 11
MQXR_TERM                                = 12
MQXR_MSG                                 = 13
MQXR_XMIT                                = 14
MQXR_SEC_MSG                             = 15
MQXR_INIT_SEC                            = 16
MQXR_RETRY                               = 17
MQXR_AUTO_CLUSSDR                        = 18
MQXR_AUTO_RECEIVER                       = 19
MQXR_CLWL_OPEN                           = 20
MQXR_CLWL_PUT                            = 21
MQXR_CLWL_MOVE                           = 22
MQXR_CLWL_REPOS                          = 23
MQXR_CLWL_REPOS_MOVE                     = 24
MQXR_END_BATCH                           = 25
MQXR_ACK_RECEIVED                        = 26
MQXR_AUTO_SVRCONN                        = 27
MQXR_AUTO_CLUSRCVR                       = 28
MQXR_SEC_PARMS                           = 29
MQXR_PUBLICATION                         = 30
MQXR_PRECONNECT                          = 31

# Exit Responses
MQXCC_OK                                 = 0
MQXCC_SUPPRESS_FUNCTION                  = (-1)
MQXCC_SKIP_FUNCTION                      = (-2)
MQXCC_SEND_AND_REQUEST_SEC_MSG           = (-3)
MQXCC_SEND_SEC_MSG                       = (-4)
MQXCC_SUPPRESS_EXIT                      = (-5)
MQXCC_CLOSE_CHANNEL                      = (-6)
MQXCC_REQUEST_ACK                        = (-7)
MQXCC_FAILED                             = (-8)

# Exit User Area Value
MQXUA_NONE                               = b"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"

# Exit User Area Value (array form)
MQXUA_NONE_ARRAY                         = [b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0'
                                          , b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0', b'\0']

# Cluster Cache Types
MQCLCT_STATIC                            = 0
MQCLCT_DYNAMIC                           = 1

# Multicast Events
MQMCEV_PACKET_LOSS                       = 1
MQMCEV_HEARTBEAT_TIMEOUT                 = 2
MQMCEV_VERSION_CONFLICT                  = 3
MQMCEV_RELIABILITY                       = 4
MQMCEV_CLOSED_TRANS                      = 5
MQMCEV_STREAM_ERROR                      = 6
MQMCEV_NEW_SOURCE                        = 10
MQMCEV_RECEIVE_QUEUE_TRIMMED             = 11
MQMCEV_PACKET_LOSS_NACK_EXPIRE           = 12
MQMCEV_ACK_RETRIES_EXCEEDED              = 13
MQMCEV_STREAM_SUSPEND_NACK               = 14
MQMCEV_STREAM_RESUME_NACK                = 15
MQMCEV_STREAM_EXPELLED                   = 16
MQMCEV_FIRST_MESSAGE                     = 20
MQMCEV_LATE_JOIN_FAILURE                 = 21
MQMCEV_MESSAGE_LOSS                      = 22
MQMCEV_SEND_PACKET_FAILURE               = 23
MQMCEV_REPAIR_DELAY                      = 24
MQMCEV_MEMORY_ALERT_ON                   = 25
MQMCEV_MEMORY_ALERT_OFF                  = 26
MQMCEV_NACK_ALERT_ON                     = 27
MQMCEV_NACK_ALERT_OFF                    = 28
MQMCEV_REPAIR_ALERT_ON                   = 29
MQMCEV_REPAIR_ALERT_OFF                  = 30
MQMCEV_RELIABILITY_CHANGED               = 31
MQMCEV_SHM_DEST_UNUSABLE                 = 80
MQMCEV_SHM_PORT_UNUSABLE                 = 81
MQMCEV_CCT_GETTIME_FAILED                = 110
MQMCEV_DEST_INTERFACE_FAILURE            = 120
MQMCEV_DEST_INTERFACE_FAILOVER           = 121
MQMCEV_PORT_INTERFACE_FAILURE            = 122
MQMCEV_PORT_INTERFACE_FAILOVER           = 123

#----------------------------------------------------------------#
# Values Related to MQ_CLUSTER_WORKLOAD_EXIT Function            #
#----------------------------------------------------------------#
# Channel exit


#- end of CMQXC -#




""" Tests for making sure MQ 8.0 features work properly.

Requires proper configuration in config.py or by setting the following
environment variables (which the config module respects) pointing to an MQ 8.0
queue managers and credentials.
Substitute values as required.

export PY_IBMMQ_TEST_QM_NAME=QM01
export PY_IBMMQ_TEST_QM_HOST=192.168.1.136
export PY_IBMMQ_TEST_QM_PORT=1434
export PY_IBMMQ_TEST_QM_CHANNEL=SVRCONN.1
export PY_IBMMQ_TEST_QM_USER=myuser
export PY_IBMMQ_TEST_QM_PASSWORD=mypassword
"""

# stdlib
import unittest

import ibmmq as mq
from ibmmq import CMQC

# test config & env
import config

# pylint: disable=no-member
class TestMQ80(unittest.TestCase):
    """The testcases"""

    def setUp(self):
        """Get elements from the overall config"""
        for key in ['NAME', 'HOST', 'PORT', 'CHANNEL', 'USER', 'PASSWORD']:
            setattr(self, key.lower(), getattr(config.MQ.QM, key))

    def get_conn(self):
        """Create a connection"""
        return mq.connect(self.name, self.channel, '{0}({1})'.format(
            self.host, self.port), self.user, self.password)

    # obviously this test can not work for a queue manager < 8.0
    @unittest.skipIf(
        int(config.MQ.QM.MIN_COMMAND_LEVEL) < 800,
        'Test only viable for a queue manager command level > 800.')
    def test_mq_level(self):
        """ We should be connecting to an MQ 8.0+ queue manager.
        """
        conn = self.get_conn()
        pcf = mq.PCFExecute(conn)
        command_level = pcf.MQCMD_INQUIRE_Q_MGR()[0][CMQC.MQIA_COMMAND_LEVEL]
        conn.disconnect()
        self.assertGreaterEqual(command_level, 800)

    def test_connect_with_credentials(self):
        """ Connecting with user credentials provided should succeed.
        """
        conn = self.get_conn()
        self.assertTrue(conn.is_connected)
        conn.disconnect()

    @unittest.skipUnless(
        config.MQ.QM.CONNAUTH.SUPPORTED == '1',
        'Test only viable for a queue manager with user/password conn auth '
        'support')
    def test_connect_with_wrong_credentials(self):
        """Modify original valid password to some bogus value"""
        bogus_password = self.password + '_Wr0nG_Pa$$w0rd'
        with self.assertRaises(mq.MQMIError) as errorcontext:
            qmgr = mq.connect(self.name, self.channel, '{0}({1})'.format(
                self.host, self.port), self.user + '_bogus_user', bogus_password)
            exception = errorcontext.exception
            self.assertEqual(exception.reason, CMQC.MQRC_NOT_AUTHORIZED)
            self.assertFalse(qmgr.is_connected)

    # The following 2 tests test_connect_without_required_credentials and
    # test_connect_without_optional_credentials are mutually exclusive and
    # intend to provide workable tests for both a (>=MQ 8.0) queue manager
    # with or without equired user/password authentication and also a (<MQ 8.0)
    # queue manager that doesn't know about user/password conn auth, anyway
    @unittest.skipUnless(
        config.MQ.QM.CONNAUTH.SUPPORTED and
        config.MQ.QM.CONNAUTH.USE_PW == 'REQUIRED',
        'Test needs a user/password-requiring queue manager')
    def test_failing_connect_without_required_credentials(self):
        """Connecting without user credentials provided should not succeed for
        a queue manager that requires user/password connection authentication.
        """

        with self.assertRaises(mq.MQMIError) as errorcontext:
            qmgr = mq.connect(self.name, self.channel, '{0}({1})'.format(
                self.host, self.port))
            exception = errorcontext.exception
            self.assertEqual(exception.reason, CMQC.MQRC_NOT_AUTHORIZED)
            self.assertFalse(qmgr.is_connected)

    @unittest.skipUnless(
        config.MQ.QM.CONNAUTH.SUPPORTED != '1' or
        config.MQ.QM.CONNAUTH.USE_PW != 'REQUIRED',
        'Test not viable for a user/password-requiring queue manager')
    def test_successful_connect_without_optional_credentials(self):
        """Connecting without user credentials should succeed for a queue
        manager that has optional user/password connection authentication.
        """
        qmgr = mq.connect(self.name, self.channel, '{0}({1})'.format(self.host, self.port))
        self.assertTrue(qmgr.is_connected)
        qmgr.disconnect()


if __name__ == "__main__":
    unittest.main()



# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example shows a simple execution of a PCF command to define a queue.
The "replace" option is used in case the object already exists. If you remove
that line and rerun the example, an error is generated.

The PCF parameters are built as a list of separate elements. Compare with the the dict-based approach
in the def_channel example.

The "admin" credentials are used to avoid needing to grant application users
additional privileges.
"""

import random
import ibmmq as mq

queue_manager = 'QM1'
channel = 'DEV.ADMIN.SVRCONN'
host = '127.0.0.1'
port = '1414'
conn_info = '%s(%s)' % (host, port)
user = 'admin'
password = 'password'

queue_name = 'MYQUEUE.1'
queue_type = mq.CMQC.MQQT_LOCAL
# Set a random max depth so it will change on each execution
max_depth = random.randint(100, 1000) * 100

args = []
args.append(mq.CFST(Parameter=mq.CMQC.MQCA_Q_NAME, String=queue_name))
args.append(mq.CFIN(Parameter=mq.CMQC.MQIA_Q_TYPE, Value=queue_type))
args.append(mq.CFIN(Parameter=mq.CMQC.MQIA_MAX_Q_DEPTH, Value=max_depth))
args.append(mq.CFIN(Parameter=mq.CMQCFC.MQIACF_REPLACE, Value=mq.CMQCFC.MQRP_YES))

qmgr = mq.connect(queue_manager, channel, conn_info, user, password)

pcf = mq.PCFExecute(qmgr)
pcf.MQCMD_CREATE_Q(args)

qmgr.disconnect()

print("Done.")



"""Test PCF usage."""
import os
from sys import version_info as sys_version_info

from unittest import skip
from unittest import skipIf
from ddt import data  # type: ignore
from ddt import ddt

try:
    from typing import List
except ImportError:
    pass

import ibmmq as mq

from test_setup import Tests  # noqa
from test_setup import main   # pylint: disable=no-name-in-module

@ddt
class TestPCF(Tests):
    """Class for MQ PCF testing."""

    messages_dir = os.path.join(os.path.dirname(__file__), "messages")

    @classmethod
    def setUpClass(cls):
        """Initialize test environment."""
        super(TestPCF, cls).setUpClass()

        # max length of queue names is 48 characters
        cls.queue_name = "{prefix}PCF.QUEUE".format(prefix=cls.prefix)

    @classmethod
    def tearDownClass(cls):
        """Tear down test environment."""
        super(TestPCF, cls).tearDownClass()

    def setUp(self):
        """Set up tesing environment."""
        super().setUp()

        self.create_queue(self.queue_name)

    def tearDown(self):
        """Delete the created objects."""
        if self.queue_name:
            self.delete_queue(self.queue_name)

        super().tearDown()

    @skip('Test not implemented')
    def test_mqcfbf(self):
        """Test MQCFBF PCF byte string filter parameter."""

    def test_mqcfbs(self):
        """Test MQCFBS PCF byte string parameter.

        Also uses MQCFIN and MQCFIL as parameters
        """
        attrs = []  # type: List[mq.MQOpts]
        attrs.append(mq.CFBS(Parameter=mq.CMQCFC.MQBACF_GENERIC_CONNECTION_ID,
                                String=b''))
        attrs.append(mq.CFIN(Parameter=mq.CMQCFC.MQIACF_CONN_INFO_TYPE,
                                Value=mq.CMQCFC.MQIACF_CONN_INFO_CONN))
        attrs.append(mq.CFIL(Parameter=mq.CMQCFC.MQIACF_CONNECTION_ATTRS,
                                Values=[mq.CMQCFC.MQIACF_ALL]))

        object_filters = []
        object_filters.append(
            mq.CFIF(Parameter=mq.CMQC.MQIA_APPL_TYPE,
                       Operator=mq.CMQCFC.MQCFOP_EQUAL,
                       FilterValue=mq.CMQC.MQAT_USER))

        results = self.pcf.MQCMD_INQUIRE_CONNECTION(attrs, object_filters)

        self.assertGreater(len(results), 0)

    def test_mqcfif(self):
        """Test string filter MQCFIF.

        Also uses MQCFST, MQCFIN and MQCFIL as parameters
        """
        attrs = []  # type: List[mq.MQOpts]
        attrs.append(mq.CFST(Parameter=mq.CMQC.MQCA_Q_NAME,
                                String=b'*'))
        attrs.append(mq.CFIN(Parameter=mq.CMQC.MQIA_Q_TYPE,
                                Value=mq.CMQC.MQQT_LOCAL))
        attrs.append(mq.CFIL(Parameter=mq.CMQCFC.MQIACF_Q_ATTRS,
                                Values=[mq.CMQC.MQIA_CURRENT_Q_DEPTH, mq.CMQC.MQCA_Q_DESC]))

        object_filters = []
        object_filters.append(
            mq.CFIF(Parameter=mq.CMQC.MQIA_CURRENT_Q_DEPTH,
                       Operator=mq.CMQCFC.MQCFOP_GREATER,
                       FilterValue=0))

        results = self.pcf.MQCMD_INQUIRE_Q(attrs, object_filters)

        self.assertTrue(results, 'Queue not found')
        for result in results:
            self.assertTrue(result[mq.CMQC.MQIA_CURRENT_Q_DEPTH] > 0,
                            'Found Queue with depth {}'.format(result[mq.CMQC.MQIA_CURRENT_Q_DEPTH]))

    def test_mqcfsf(self):
        """Test string filter MQCFSF.

        Also uses MQCFST, MQCFIN and MQCFIL as parameters
        """
        attrs = []  # type: List[mq.MQOpts]
        attrs.append(mq.CFST(Parameter=mq.CMQC.MQCA_Q_NAME,
                                String=b'*'))
        attrs.append(mq.CFIN(Parameter=mq.CMQC.MQIA_Q_TYPE,
                                Value=mq.CMQC.MQQT_LOCAL))
        attrs.append(mq.CFIL(Parameter=mq.CMQCFC.MQIACF_Q_ATTRS,
                                Values=[mq.CMQC.MQIA_CURRENT_Q_DEPTH, mq.CMQC.MQCA_Q_DESC]))

        object_filters = []
        object_filters.append(
            mq.CFSF(Parameter=mq.CMQC.MQCA_Q_DESC,
                       Operator=mq.CMQCFC.MQCFOP_LIKE,
                       FilterValue=b'IBM MQ*'))

        results = self.pcf.MQCMD_INQUIRE_Q(attrs, object_filters)

        self.assertTrue(results, 'Queue not found')
        for result in results:
            self.assertTrue(not result[mq.CMQC.MQCA_Q_DESC].startswith(b'MQ'),
                            'Found Queue with description {}'.format(result[mq.CMQC.MQCA_Q_DESC]))
            self.assertTrue(mq.CMQC.MQCA_Q_DESC in result,
                            'Attribute {} is not returned'.format(result[mq.CMQC.MQCA_Q_DESC]))

    @data([], [b'One'], [b'One', b'Two', b'Three'])
    def test_mqcfsl(self, value):
        """Test MQCFSL PCF string list parameter.

        Also uses MQCFST and MQCFIN as parameters
        """
        attrs = []  # type: List[mq.MQOpts]
        attrs.append(mq.CFST(Parameter=mq.CMQC.MQCA_NAMELIST_NAME,
                                String='{}NAMELIST'.format(self.prefix).encode()))
        attrs.append(mq.CFSL(Parameter=mq.CMQC.MQCA_NAMES,
                                Strings=value))
        attrs.append(mq.CFIN(Parameter=mq.CMQCFC.MQIACF_REPLACE,
                                Value=mq.CMQCFC.MQRP_YES))

        try:
            self.pcf.MQCMD_CREATE_NAMELIST(attrs)
        except Exception:  # pylint: disable=broad-except
            self.fail('Exception occurs!')
        else:
            attrs = []
            attrs.append(mq.CFST(Parameter=mq.CMQC.MQCA_NAMELIST_NAME,
                                    String='{}NAMELIST'.format(self.prefix).encode()))
            attrs.append(mq.CFIL(Parameter=mq.CMQCFC.MQIACF_NAMELIST_ATTRS,
                                    Values=[mq.CMQC.MQCA_NAMES, mq.CMQC.MQIA_NAME_COUNT]))

            results = self.pcf.MQCMD_INQUIRE_NAMELIST(attrs)

            self.assertEqual(results[0][mq.CMQC.MQIA_NAME_COUNT], len(value))

            if results[0][mq.CMQC.MQIA_NAME_COUNT] > 0:
                for item in results[0][mq.CMQC.MQCA_NAMES]:
                    item = item.strip()
                    self.assertTrue(item in value, '{} value not in values list'.format(item))
                    value.remove(item)

            attrs = []
            attrs.append(mq.CFST(Parameter=mq.CMQC.MQCA_NAMELIST_NAME,
                                    String='{}NAMELIST'.format(self.prefix).encode()))
            self.pcf.MQCMD_DELETE_NAMELIST(attrs)

    @data([], [1], [1, 2, 3, 4, 5])
    def test_arbitrary_message_with_mqcfil(self, value):
        """Test arbitrary message with MQCFIL."""
        message = mq.CFH(Version=mq.CMQCFC.MQCFH_VERSION_1,
                            Type=mq.CMQCFC.MQCFT_USER,
                            ParameterCount=1).pack()
        message = message + mq.CFIL(Parameter=1,
                                       Values=value).pack()

        queue = mq.Queue(self.qmgr, self.queue_name,
                            mq.CMQC.MQOO_INPUT_AS_Q_DEF + mq.CMQC.MQOO_OUTPUT)

        put_md = mq.MD(Format=mq.CMQC.MQFMT_PCF)
        queue.put(message, put_md)

        get_opts = mq.GMO(
            Options=mq.CMQC.MQGMO_NO_SYNCPOINT + mq.CMQC.MQGMO_FAIL_IF_QUIESCING,
            Version=mq.CMQC.MQGMO_VERSION_2,
            MatchOptions=mq.CMQC.MQMO_MATCH_CORREL_ID)
        get_md = mq.MD(MsgId=put_md.MsgId)  # pylint: disable=no-member
        message = queue.get(None, get_md, get_opts)
        queue.close()
        unpacked_message = mq.PCFExecute.unpack(message)

        self.assertTrue(isinstance(unpacked_message[0][1], list),
                        'Returned value is not list: {}'.format(type(unpacked_message[0][1])))

        self.assertTrue(len(unpacked_message[0][1]) == len(value), 'List length is different!')

        for item in unpacked_message[0][1]:
            self.assertTrue(item in value, '{} value not in values list'.format(item))
            value.remove(item)

    def test_mqcfgr_mqcfin64_mqcfil64(self):
        """Test arbitrary message with MQCFIL."""

        # Groups require CFH_VERSION_3
        cfh = mq.CFH(Version=mq.CMQCFC.MQCFH_VERSION_3,
                            Type=mq.CMQCFC.MQCFT_USER,
                            ParameterCount=4)
        message = cfh.pack()
        message += mq.CFST(Parameter=mq.CMQC.MQCA_Q_MGR_NAME,
                              String=b'QM1').pack()
        # group1
        message += mq.CFGR(Parameter=mq.CMQCFC.MQGACF_Q_STATISTICS_DATA,
                              ParameterCount=3).pack()
        message += mq.CFST(Parameter=mq.CMQC.MQCA_Q_NAME,
                              String=b'SYSTEM.ADMIN.COMMAND.QUEUE').pack()
        message += mq.CFIN64(Parameter=mq.CMQCFC.MQIAMO_Q_MIN_DEPTH,
                                Value=10).pack()
        message += mq.CFIL64(Parameter=mq.CMQCFC.MQIAMO64_AVG_Q_TIME,
                                Values=[1, 2, 3]).pack()
        # group2
        message += mq.CFGR(Parameter=mq.CMQCFC.MQGACF_Q_STATISTICS_DATA,
                              ParameterCount=3).pack()
        message += mq.CFST(Parameter=mq.CMQC.MQCA_Q_NAME,
                              String=b'SYSTEM.ADMIN.COMMAND.QUEUE2').pack()
        message += mq.CFIN64(Parameter=mq.CMQCFC.MQIAMO_Q_MIN_DEPTH,
                                Value=20).pack()
        message += mq.CFIL64(Parameter=mq.CMQCFC.MQIAMO64_AVG_Q_TIME,
                                Values=[111, 222]).pack()

        message += mq.CFST(Parameter=mq.CMQCFC.MQCAMO_START_TIME,
                              String=b'10.41.58').pack()

        queue = mq.Queue(self.qmgr, self.queue_name,
                            mq.CMQC.MQOO_INPUT_AS_Q_DEF + mq.CMQC.MQOO_OUTPUT)

        put_md = mq.MD(Format=mq.CMQC.MQFMT_PCF)
        queue.put(message, put_md)

        get_opts = mq.GMO(
            Options=mq.CMQC.MQGMO_NO_SYNCPOINT + mq.CMQC.MQGMO_FAIL_IF_QUIESCING,
            Version=mq.CMQC.MQGMO_VERSION_2,
            MatchOptions=mq.CMQC.MQMO_MATCH_CORREL_ID)
        get_md = mq.MD(MsgId=put_md.MsgId)  # pylint: disable=no-member
        message = queue.get(None, get_md, get_opts)
        queue.close()
        message, _ = mq.PCFExecute.unpack(message)

        self.assertEqual({
            mq.CMQC.MQCA_Q_MGR_NAME: b'QM1\x00',
            mq.CMQCFC.MQCAMO_START_TIME: b'10.41.58',
            mq.CMQCFC.MQGACF_Q_STATISTICS_DATA: [
                {
                    mq.CMQC.MQCA_Q_NAME: b'SYSTEM.ADMIN.COMMAND.QUEUE\x00\x00',
                    mq.CMQCFC.MQIAMO_Q_MIN_DEPTH: 10,
                    mq.CMQCFC.MQIAMO64_AVG_Q_TIME: [1, 2, 3],
                },
                {
                    mq.CMQC.MQCA_Q_NAME: b'SYSTEM.ADMIN.COMMAND.QUEUE2\x00',
                    mq.CMQCFC.MQIAMO_Q_MIN_DEPTH: 20,
                    mq.CMQCFC.MQIAMO64_AVG_Q_TIME: [111, 222],
                },
            ]
        }, message)

    def test_unpack_header(self):
        """Test unpack header."""
        message = mq.CFH(Version=mq.CMQCFC.MQCFH_VERSION_1,
                            Type=mq.CMQCFC.MQCFT_STATISTICS,
                            Command=mq.CMQCFC.MQCMD_STATISTICS_Q,
                            ParameterCount=1).pack()
        message += mq.CFST(Parameter=mq.CMQC.MQCA_Q_MGR_NAME,
                              String=b'QM1').pack()

        queue = mq.Queue(self.qmgr, self.queue_name,
                            mq.CMQC.MQOO_INPUT_AS_Q_DEF + mq.CMQC.MQOO_OUTPUT)

        put_md = mq.MD(Format=mq.CMQC.MQFMT_PCF)
        queue.put(message, put_md)

        get_opts = mq.GMO(
            Options=mq.CMQC.MQGMO_NO_SYNCPOINT + mq.CMQC.MQGMO_FAIL_IF_QUIESCING,
            Version=mq.CMQC.MQGMO_VERSION_2,
            MatchOptions=mq.CMQC.MQMO_MATCH_CORREL_ID)
        get_md = mq.MD(MsgId=put_md.MsgId)  # pylint: disable=no-member
        message = queue.get(None, get_md, get_opts)
        queue.close()
        message, header = mq.PCFExecute.unpack(message)

        self.assertEqual(header.Command, mq.CMQCFC.MQCMD_STATISTICS_Q)  # pylint: disable=no-member
        self.assertEqual(header.Type, mq.CMQCFC.MQCFT_STATISTICS)  # pylint: disable=no-member

        self.assertEqual({
            mq.CMQC.MQCA_Q_MGR_NAME: b'QM1\x00',
        }, message)

    def test_unpack_group(self):
        """Test parameters group unpack."""
        with open(os.path.join(self.messages_dir, "statistics_q.dat"), "rb") as file:
            binary_message = file.read()
            message, header = mq.PCFExecute.unpack(binary_message)

            self.assertEqual(header.Command, mq.CMQCFC.MQCMD_STATISTICS_Q)  # pylint: disable=no-member
            self.assertEqual(header.Type, mq.CMQCFC.MQCFT_STATISTICS)  # pylint: disable=no-member

            self.assertEqual(message[mq.CMQC.MQCA_Q_MGR_NAME].strip(), b'mq_mgr1')
            self.assertEqual(message[mq.CMQCFC.MQCAMO_START_DATE], b'2020-06-15\x00\x00')
            self.assertEqual(len(message[mq.CMQCFC.MQGACF_Q_STATISTICS_DATA]), 16)

            item = message[mq.CMQCFC.MQGACF_Q_STATISTICS_DATA][0]
            self.assertEqual(item[mq.CMQC.MQCA_Q_NAME].strip(), b'SYSTEM.ADMIN.COMMAND.QUEUE')
            self.assertEqual(item[mq.CMQCFC.MQIAMO_PUTS], [14, 0])

    def test_unpack_cfsf(self):
        """Test unpack of PCF message with MQCFSF structure."""
        with open(os.path.join(self.messages_dir, "pcf_with_cfsf.dat"), "rb") as file:
            binary_message = file.read()

            message, _ = mq.PCFExecute.unpack(binary_message)

        self.assertEqual(message.get(mq.CMQCFC.MQGACF_COMMAND_DATA, [{}])[0].get(mq.CMQC.MQCA_Q_DESC)[0],
                         mq.CMQCFC.MQCFOP_LIKE)

        self.assertEqual(message.get(mq.CMQCFC.MQGACF_COMMAND_DATA,
                                     [{}])[0].get(mq.CMQC.MQCA_Q_DESC)[1].rstrip(b'\x00'),
                         b'test*')

    @skip('Test not implemented')
    def test_unpack_cfbf(self):
        """Test unpack of PCF message with MQCFBF structure."""

    def test_unpack_cfif(self):
        """Test unpack of PCF message with MQCFIF structure."""
        with open(os.path.join(self.messages_dir, "pcf_with_cfif.dat"), "rb") as file:
            binary_message = file.read()

            message, _ = mq.PCFExecute.unpack(binary_message)

        self.assertEqual(message.get(mq.CMQCFC.MQGACF_COMMAND_DATA, [{}])[0].get(mq.CMQC.MQIA_CURRENT_Q_DEPTH),
                         (mq.CMQCFC.MQCFOP_GREATER, 0))

    @skipIf(sys_version_info < (3, 7), 'Python pre 3.7 issues')
    def test_mqcfbs_old(self):
        """Test byte string MQCFBS and MQCFIL with old style."""
        attrs = {
            mq.CMQCFC.MQBACF_GENERIC_CONNECTION_ID: mq.ByteString(b''),
            mq.CMQCFC.MQIACF_CONN_INFO_TYPE: mq.CMQCFC.MQIACF_CONN_INFO_CONN,
            mq.CMQCFC.MQIACF_CONNECTION_ATTRS: [mq.CMQCFC.MQCACH_CONNECTION_NAME,
                                                   mq.CMQCFC.MQCACH_CHANNEL_NAME]
        }
        fltr = mq.Filter(mq.CMQC.MQIA_APPL_TYPE).equal(mq.CMQC.MQAT_USER)

        results = self.pcf.MQCMD_INQUIRE_CONNECTION(attrs, [fltr])

        self.assertGreater(len(results), 0)
        self.assertTrue(mq.CMQCFC.MQCACH_CONNECTION_NAME in results[0].keys())
        self.assertTrue(mq.CMQCFC.MQCACH_CHANNEL_NAME in results[0].keys())

    @skipIf(sys_version_info < (3, 7), 'Python pre 3.7 issues')
    @data(['test1'],
          ['test1', b'test2'],
          [b'test1', 'test2'])
    def test_mqcfsl_old(self, names):
        """Test list MQCFSL with old style."""

        args = {
            mq.CMQC.MQCA_NAMELIST_NAME: "{prefix}PCF.NAMELIST".format(prefix=self.prefix),
            mq.CMQC.MQCA_NAMES: names,
            mq.CMQCFC.MQIACF_REPLACE: mq.CMQCFC.MQRP_YES
        }

        self.pcf.MQCMD_CREATE_NAMELIST(args)

        args = {
            mq.CMQC.MQCA_NAMELIST_NAME: "{prefix}PCF.NAMELIST".format(prefix=self.prefix),
            mq.CMQCFC.MQIACF_NAMELIST_ATTRS: [mq.CMQC.MQCA_NAMES]
        }
        results = None
        try:
            results = self.pcf.MQCMD_INQUIRE_NAMELIST(args)
        finally:
            args = {
                mq.CMQC.MQCA_NAMELIST_NAME: "{prefix}PCF.NAMELIST".format(prefix=self.prefix),
            }

            self.pcf.MQCMD_DELETE_NAMELIST(args)
        for name in names:
            self.assertTrue(mq.ensure_bytes(name) in [x.strip() for x in results[0][mq.CMQC.MQCA_NAMES]])

    @data(mq.CMQCFC.MQIACF_ALL, [mq.CMQCFC.MQIACF_ALL],
          mq.CMQC.MQCA_Q_DESC, [mq.CMQC.MQCA_Q_DESC],
          [mq.CMQC.MQIA_CURRENT_Q_DEPTH, mq.CMQC.MQCA_Q_DESC])
    def test_object_filter_int_old_queue(self, value):
        """Test object filter with integer attribute. Old style."""
        attrs = {
            mq.CMQC.MQCA_Q_NAME: b'*',
            mq.CMQCFC.MQIACF_Q_ATTRS: value
            }

        filter_depth = mq.Filter(mq.CMQC.MQIA_CURRENT_Q_DEPTH).greater(0)

        results = self.pcf.MQCMD_INQUIRE_Q(attrs, [filter_depth])

        self.assertTrue(results, 'Queue not found')
        for result in results:
            self.assertTrue(result[mq.CMQC.MQIA_CURRENT_Q_DEPTH] > 0,
                            'Found Queue with depth {}'.format(result[mq.CMQC.MQIA_CURRENT_Q_DEPTH]))

    @skip('https://stackoverflow.com/questions/62250844/ibm-mq-pcf-parameters-order')
    @data(mq.CMQCFC.MQIACF_ALL, [mq.CMQCFC.MQIACF_ALL],
          mq.CMQCFC.MQCACH_DESC, [mq.CMQCFC.MQCACH_DESC],
          [mq.CMQCFC.MQCACH_DESC, mq.CMQCFC.MQIACH_CHANNEL_TYPE])
    def test_object_filter_int_old_channel(self, value):
        """Test object filter with integer attribute. Old style."""
        attrs = {
            mq.CMQCFC.MQCACH_CHANNEL_NAME: b'*',
            mq.CMQCFC.MQIACF_CHANNEL_ATTRS: value}

        filter_type = mq.Filter(mq.CMQCFC.MQIACH_CHANNEL_TYPE).equal(mq.CMQXC.MQCHT_SVRCONN)

        results = self.pcf.MQCMD_INQUIRE_CHANNEL(attrs, [filter_type])

        self.assertTrue(results, 'Channel not found')
        for result in results:
            self.assertTrue(result[mq.CMQCFC.MQIACH_CHANNEL_TYPE] == mq.CMQXC.MQCHT_SVRCONN,
                            'Found Channel with type {}'.format(result[mq.CMQCFC.MQIACH_CHANNEL_TYPE]))

    def test_object_filter_str_old(self):
        """Test object filter with string attribute. Old style."""
        attrs = {
            mq.CMQC.MQCA_Q_NAME: b'*',
            mq.CMQCFC.MQIACF_Q_ATTRS: [mq.CMQC.MQIA_CURRENT_Q_DEPTH, mq.CMQC.MQCA_Q_DESC]
            }

        filter_depth = mq.Filter(mq.CMQC.MQCA_Q_DESC).like(b'IBM MQ *')

        results = self.pcf.MQCMD_INQUIRE_Q(attrs, [filter_depth])

        self.assertTrue(results, 'Queue not found')
        for result in results:
            self.assertTrue(not result[mq.CMQC.MQCA_Q_DESC].startswith(b'MQ'),
                            'Found Queue with description {}'.format(result[mq.CMQC.MQCA_Q_DESC]))

    def test_disconnect(self):
        """Test disconnect for PCF object."""
        # pylint: disable=protected-access

        pcf = mq.PCFExecute(self.qmgr)

        self.assertTrue(pcf.reply_queue)
        self.assertTrue(pcf.reply_queue_name)

        pcf.disconnect()

        self.assertTrue(self.qmgr)
        self.assertFalse(pcf.reply_queue)
        self.assertFalse(pcf.reply_queue_name)


if __name__ == "__main__":
    main(module="test_pcf")



"""MQOD: Object Descriptor"""

# Copyright (c) 2025 IBM Corporation and other Contributors. All Rights Reserved.
# Copyright (c) 2009-2024 Dariusz Suchojad. All Rights Reserved.

from mqcommon import *
from mqopts import MQOpts
from ibmmq import CMQC, ibmmqc

# This would have to change for running on a z/OS system where
# the dynamic queue prefix is "CSQ.*"
_DEFAULT_DQ_PREFIX = b'AMQ.*'

class OD(MQOpts):
    """ Construct an MQOD Structure with default values as per MQI.
    The default values may be overridden by the optional keyword arguments 'kw'.
    """
    def __init__(self, **kw):
        opts = [['_StrucId', CMQC.MQOD_STRUC_ID, '4s'],
                ['Version', CMQC.MQOD_VERSION_4, MQLONG_TYPE],
                ['ObjectType', CMQC.MQOT_Q, MQLONG_TYPE],
                ['ObjectName', b'', '48s'],
                ['ObjectQMgrName', b'', '48s'],
                ['DynamicQName', _DEFAULT_DQ_PREFIX, '48s'],
                ['AlternateUserId', b'', '12s'],
                ['RecsPresent', 0, MQLONG_TYPE],
                ['KnownDestCount', 0, MQLONG_TYPE],
                ['UnknownDestCount', 0, MQLONG_TYPE],
                ['InvalidDestCount', 0, MQLONG_TYPE],
                ['ObjectRecOffset', 0, MQLONG_TYPE],
                ['ResponseRecOffset', 0, MQLONG_TYPE],
                ['ObjectRecPtr', 0, 'P'],
                ['ResponseRecPtr', 0, 'P'],
                ['AlternateSecurityId', b'', '40s'],
                ['ResolvedQName', b'', '48s'],
                ['ResolvedQMgrName', b'', '48s'], ]

        od_current_version = ibmmqc.__strucversions__.get("od", 1)
        if od_current_version >= CMQC.MQOD_VERSION_4:
            opts += [

                # ObjectString
                ['ObjectStringVSPtr', 0, 'P'],
                ['ObjectStringVSOffset', (0), MQLONG_TYPE],
                ['ObjectStringVSBufSize', (0), MQLONG_TYPE],
                ['ObjectStringVSLength', (0), MQLONG_TYPE],
                ['ObjectStringVSCCSID', (0), MQLONG_TYPE],

                # SelectionString
                ['SelectionStringVSPtr', 0, 'P'],
                ['SelectionStringVSOffset', (0), MQLONG_TYPE],
                ['SelectionStringVSBufSize', (0), MQLONG_TYPE],
                ['SelectionStringVSLength', (0), MQLONG_TYPE],
                ['SelectionStringVSCCSID', (0), MQLONG_TYPE],

                # ResObjectString
                ['ResObjectStringVSPtr', 0, 'P'],
                ['ResObjectStringVSOffset', (0), MQLONG_TYPE],
                ['ResObjectStringVSBufSize', (0), MQLONG_TYPE],
                ['ResObjectStringVSLength', (0), MQLONG_TYPE],
                ['ResObjectStringVSCCSID', (0), MQLONG_TYPE],

                ['ResolvedType', (-3), MQLONG_TYPE]]

            # For 64bit platforms MQLONG is an int and this pad
            # needs to be here for MQ 7.0
            if MQLONG_TYPE == 'i':
                opts += [['pad', b'', '4s']]

        super().__init__(tuple(opts), **kw)


# Backward compatibility
od = OD  # pylint: disable=invalid-name



"""Use the environment to get configuration values for connection to qmgr"""
import os.path
import utils


class PATHS:
    """Location of testcases"""
    TESTS_DIR = os.path.normpath(os.path.dirname(__file__))


class MQ:
    """
    Configuration for the tests
    By default, inherited from tox.ini
    """
    class QM:
        """Queue manager connection setup"""
        NAME = os.environ.get('PY_IBMMQ_TEST_QM_NAME', 'QM1')
        HOST = os.environ.get('PY_IBMMQ_TEST_QM_HOST', 'localhost')
        PORT = os.environ.get('PY_IBMMQ_TEST_QM_PORT', '1413')
        TRANSPORT = os.environ.get('PY_IBMMQ_TEST_QM_TRANSPORT', 'TCP')

        CHANNEL = os.environ.get('PY_IBMMQ_TEST_QM_CHANNEL', 'DEV.ADMIN.SVRCONN')
        USER = os.environ.get('PY_IBMMQ_TEST_QM_USER', 'admin')
        PASSWORD = os.environ.get('PY_IBMMQ_TEST_QM_PASSWORD', 'password')

        APP_CHANNEL = os.environ.get('PY_IBMMQ_TEST_QM_APP_CHANNEL', 'DEV.APP.SVRCONN')
        APP_USER = os.environ.get('PY_IBMMQ_TEST_QM_APP_USER', 'app')
        APP_PASSWORD = os.environ.get('PY_IBMMQ_TEST_QM_APP_PASSWORD', 'password')

        MIN_COMMAND_LEVEL = os.environ.get(
            'PY_IBMMQ_TEST_QM_MIN_COMMAND_LEVEL', '800')

        class CONNAUTH:
            """Configuration of authentication credentials"""
            # user/password connection authentication is a MQ >= 8.0 feature
            SUPPORTED = os.environ.get('PY_IBMMQ_TEST_QM_CONNAUTH_SUPPORTED',
                                       '1')
            # Set to OPTIONAL or REQUIRED
            # OPTIONAL: If a user ID and password are provided by a client
            # application then they must be a valid pair. It is not mandatory
            # to provide user + password, though.
            # REQUIRED: A valid user ID and password are mandatory.
            USE_PW = os.environ.get(
                'PY_IBMMQ_TEST_QM_CONNAUTH_USE_PW', 'REQUIRED')

            # Delay time in seconds for API call returns in case of auth
            # failures (some DoS-countermeasure). For testing purposes we
            # usually want this as fast as possible. This value gets used in
            # create_mq_objects.py for the creation of the queue manager conn
            # auth.
            FAIL_DELAY = os.environ.get(
                'PY_IBMMQ_TEST_QM_CONNAUTH_FAIL_DELAY', '0')

    class QUEUE:
        """Queue naming setup"""
        PREFIX = os.environ.get('PY_IBMMQ_TEST_QUEUE_PREFIX', 'UNITTEST')
        QUEUE_NAMES = {
            'TestRFH2PutGet': PREFIX + 'TEST.PYIBMMQ.RFH2PUTGET',
            'TestQueueManager': PREFIX + 'TEST.PYIBMMQ.QUEUEMANAGER',
            }

    # convenience attribute derived from above settings, may be used for tests
    # that mandate the MQSERVER environment variable
    # E.g. MQSERVER="SVRCONN.1/TCP/mq.example.org(1777)"
    MQSERVER = '%(channel)s/%(transport)s/%(host)s(%(port)s)' % {
        'channel': QM.CHANNEL,
        'transport': QM.TRANSPORT,
        'host': QM.HOST,
        'port': QM.PORT,
        }

    APP_MQSERVER = '%(channel)s/%(transport)s/%(host)s(%(port)s)' % {
        'channel': QM.APP_CHANNEL,
        'transport': QM.TRANSPORT,
        'host': QM.HOST,
        'port': QM.PORT,
        }


if __name__ == '__main__':
    utils.print_config(PATHS, MQ)



# More examples are at https://github.com/ibm-messaging/mq-dev-patterns
# and in code/examples in the source distribution.

"""
This example shows a simple execution of a PCF command to define a channel.
The "replace" option is used in case the object already exists. If you remove
that line and rerun the example, an error is generated.

The PCF parameters are passed as a dictionary of attribute name and value pairs. Compare with
the style used in the def_queue example.

The "admin" credentials are used to avoid needing to grant application users
additional privileges.
"""

import ibmmq as mq

queue_manager = 'QM1'
channel = 'DEV.ADMIN.SVRCONN'
host = '127.0.0.1'
port = '1414'
conn_info = '%s(%s)' % (host, port)
user = 'admin'
password = 'password'

channel_name = 'MYCHANNEL.1'
channel_type = mq.CMQXC.MQCHT_SVRCONN

args = {
    mq.CMQCFC.MQCACH_CHANNEL_NAME: channel_name,
    mq.CMQCFC.MQIACH_CHANNEL_TYPE: channel_type,
    mq.CMQCFC.MQIACF_REPLACE: mq.CMQCFC.MQRP_YES
}

qmgr = mq.connect(queue_manager, channel, conn_info, user, password)

pcf = mq.PCFExecute(qmgr)
pcf.MQCMD_CREATE_CHANNEL(args)

qmgr.disconnect()

print("Done.")
